//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.tool.configurator;

import fipaos.agent.profile.*;
import fipaos.agent.dataMapper.*;
import fipaos.util.*;

import java.awt.*;
import java.io.File;
import java.util.*;
import javax.swing.*;
import javax.swing.filechooser.*;

public class FIPAOSConfigurator
{
	
	private static final String MARKER 	= "FIPA_OS_Installation_Marker.txt";
	private static final String PROFILES 	= "profiles";
	
	private static final String PLATFORM_PROFILE 	= "platform.profile";
	private static final String ACC_PROFILE 	= "acc.profile";
	private static final String DEFAULT_PROFILE 	= "default.profile";
	private static final String LOADER_PROFILE 	= "loader.profile";
	private static final String RUNTIME_PROFILE 	= "SetupFIPAOS";
	
	private String _platform_profile_location;
	private String _acc_profile_location;
	private String _default_profile_location;
	private String _loader_profile_location;
	private String _runtime_profile_location;
	
	private String _true_fipaos_location;
	
	private static final String _separator = System.getProperty("file.separator");
	
	public FIPAOSConfigurator(String fipa_os_location)
	{
	
		_true_fipaos_location = locateFIPAOS(fipa_os_location);
		
		if (_true_fipaos_location == null)
		{
			System.out.println("Could not locate your FIPA-OS installation directory");
			System.exit(-1);
		}
		
		String profiles_dir = _true_fipaos_location + _separator + PROFILES + _separator;
		
		boolean first_run = false;
		
		// Fetch platform profile
		_platform_profile_location 	 = profiles_dir + PLATFORM_PROFILE;
		PlatformProfile platform_profile = (PlatformProfile)fetchProfile(_platform_profile_location, PLATFORM_PROFILE);
		
		// Check if platform profile has a profile-location specified
		if( platform_profile.getProfileDirectory() == null ||
		platform_profile.getProfileDirectory().equals( "" ) )
		{
			// Set profile location to be the same directory as the platform profile itself
			platform_profile.setProfileDirectory( profiles_dir );
			
			first_run = true;
		}
		else
		{
                    try
                    {
                            // Update profiles location appropriately
                            profiles_dir = new File(platform_profile.getProfileDirectory()).getCanonicalPath() + _separator;
                    }
                    catch( Throwable t )
                    {
                            DIAGNOSTICS.println( "Error reading profile - aborting", t, DIAGNOSTICS.LEVEL_MAX );
                            return;
                    }
		}
		
		_acc_profile_location		= profiles_dir + ACC_PROFILE;
		_default_profile_location	= profiles_dir + DEFAULT_PROFILE;
		_loader_profile_location	= profiles_dir + LOADER_PROFILE;
		_runtime_profile_location	= _true_fipaos_location + _separator + "bat" + _separator + RUNTIME_PROFILE;
		
		ACCProfile acc_profile		= (ACCProfile)fetchProfile(_acc_profile_location, ACC_PROFILE);
		AgentProfile default_profile	= (AgentProfile)fetchProfile(_default_profile_location, DEFAULT_PROFILE);
		LoaderProfile loader_profile	= (LoaderProfile)fetchProfile(_loader_profile_location, LOADER_PROFILE);

		if (first_run == true)
		{
			if (acc_profile != null)
			{
				acc_profile.setLocalAddressesLocation(_true_fipaos_location + _separator + "platform.addresses");
				acc_profile.getDatabaseProfile().setDatabaseLocation(_true_fipaos_location + _separator + "databases" + _separator);
			}
			if (default_profile != null)
			{
				default_profile.getDatabaseProfile().setDatabaseLocation(_true_fipaos_location + _separator + "databases" + _separator);
			}
		}
		
		System.out.println("Loading Runtime Profile");
		
		RuntimeProfile runtime_profile	= (RuntimeProfile)fetchProfile(_runtime_profile_location, RUNTIME_PROFILE);
		
		System.out.println("Starting GUI");
		FIPAOSConfiguratorGUI gui = new FIPAOSConfiguratorGUI(
		this,
		platform_profile,
		acc_profile,
		default_profile,
		loader_profile,
		runtime_profile);
		
	} // end FIPAOSConfigurator2()
	
	public void saveProfile(Profile profile) throws Exception
	{
		DataMapper dataMapper = null;
		
		if (profile instanceof PlatformProfile)
		{
			dataMapper = DataMapperFactory.createDataMapperByFile( _platform_profile_location, null );
		}
		else if ( profile instanceof ACCProfile)
		{
			dataMapper = DataMapperFactory.createDataMapperByFile( _acc_profile_location, null );
		}
		else if (profile instanceof AgentProfile)
		{
			dataMapper = DataMapperFactory.createDataMapperByFile( _default_profile_location, null );
		}
		else if (profile instanceof LoaderProfile)
		{
			dataMapper = DataMapperFactory.createDataMapperByFile( _loader_profile_location, null );
		}
		
		else if (profile instanceof RuntimeProfile)
		{
			dataMapper = DataMapperFactory.createDataMapperByFile( _runtime_profile_location, null );
		}
		
		if( dataMapper != null )
			dataMapper.marshall( profile );
		
	} // saveProfile()
	
	private Profile fetchProfile(String profile_location, String filename)
	{
		Profile profile = null;
		
		// if profile location is null then search was selected for a
		// missing profile
		if (profile_location ==  null)
		{
			
			JFileChooser chooser = new JFileChooser(_true_fipaos_location);
			
			chooser.setDialogTitle("Select location of " + filename);
			
			int returnVal = chooser.showDialog(null, "Select");
			
			if(returnVal == JFileChooser.APPROVE_OPTION)
			{
				
				profile_location = chooser.getSelectedFile().getAbsolutePath();
				
			}
			
		} // end if profile_location == null
		
		try
		{
			DataMapper dataMapper = DataMapperFactory.createDataMapperByFile( profile_location, null );			
			if( dataMapper != null )
				profile = (Profile) dataMapper.unmarshall();
		}
		catch (Throwable t)
		{
			profile = null;
		}
		
		if (profile == null)
		{
			Object[] options =
			{ "Search", "Create New" };
			
			int result = JOptionPane.showOptionDialog(
			null,
			"Could not locate " + filename,
			"Warning",
			JOptionPane.DEFAULT_OPTION,
			JOptionPane.WARNING_MESSAGE,
			null,
			options,
			options[0]);
			
			if (result == 0) // search chosen
			{
				return fetchProfile(null, filename);
			}
			else // new chosen
			{
				return null;
			}
		}
		else
		{
			return profile;
		}
		
	} // end fetchProfile()
	
	private String locateFIPAOS(String fipa_os_location)
	{
		
		boolean found_fipa_os;
		
		File fipa_os_dir = null;
		
		if (fipa_os_location == null)
		{
			JFileChooser chooser = new JFileChooser();
			
			chooser.setDialogTitle("Select FIPA-OS installation directory");
			
			chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
			chooser.setFileFilter(new DirFilter());
			
			int returnVal = chooser.showDialog(null, "Select");
			
			if(returnVal == JFileChooser.APPROVE_OPTION)
			{
				fipa_os_dir = chooser.getSelectedFile();
				System.out.println("Looking for installation in : " + fipa_os_dir);
			}
			
		}
		else
		{
			fipa_os_dir = new File(fipa_os_location);
		}
		
		found_fipa_os = verifyLocation(fipa_os_dir);
		
		if (found_fipa_os == true)
		{
			
			try
			{
				return fipa_os_dir.getCanonicalPath();
			}
			catch (Throwable t)
			{
				return null;
			}
		}
		
		return null;
		
	} // end locateFIPAOS()
	
	private boolean verifyLocation(File location)
	{
		if (location == null)
		{
			return false;
		}
		
		String contents[] = location.list();
		
		if (contents == null)
		{
			return false;
		}
		
		for (int i = 0; i < contents.length; i++)
		{
			if (contents[i].equals(MARKER))
			{
				return true;
			}
		} // end for
		
		return false;
		
	} // end verifyLocation()
	
	private class DirFilter extends FileFilter
	{
		
		public boolean accept(File file)
		{
			return file.isDirectory();
		}
		
		public String getDescription()
		{
			return "Directories";
		}
		
	} // end DirFilter
	
	public static void main(String args[])
	{
		if (args.length > 0)
		{
			FIPAOSConfigurator config = new FIPAOSConfigurator(args[0]);
		}
		else
		{
			FIPAOSConfigurator config = new FIPAOSConfigurator(null);
		}
		
	} // end main()
	
} // end FIPAOSConfigurator.java