//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.tool.configurator;

import fipaos.agent.profile.*;
import fipaos.util.DIAGNOSTICS;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;

import java.util.*;

/****************************************************************************
 *
 * FIPAOSConfiguratorGUI
 *
 * This is the gui part of the configuration tool.
 * It handles the display and editing of the configuration variables.
 *
 ****************************************************************************/
public class FIPAOSConfiguratorGUI extends JFrame implements ActionListener, WindowListener
{
	
	private PlatformProfile _platform_profile;
	private ACCProfile _acc_profile;
	private AgentProfile _default_profile;
	private LoaderProfile _loader_profile;	
	private RuntimeProfile _runtime_profile;
	
	private TaskManagerProfile _tm_profile;	
	
	private static final String title = "FIPA-OS Configuration Tool";
	
	private static String ACC_PROFILE 	= "ACC Profile";
	private static String PLATFORM_PROFILE 	= "Platform Profile";
	private static String LOADER_PROFILE 	= "Loader Profile";	
	private static String RUNTIME_PROFILE 	= "Runtime Profile";
	private static String DEFAULT_PROFILE 	= "Default Agent Profile";
	
	private static String ADDING 	= "Adding";
	private static String EDITING 	= "Editing";
	
	private static final int ADD_REMOTE_PROFILE 		= 0;
	private static final int EDIT_REMOTE_PROFILE 		= 1;
	private static final int ADD_INTERNAL_TRANSPORT		= 2;
	private static final int EDIT_INTERNAL_TRANSPORT	= 3;
	private static final int ADD_EXTERNAL_TRANSPORT		= 4;
	private static final int EDIT_EXTERNAL_TRANSPORT	= 5;
	
	private static final int ACC_PANE 			= 0;
	private static final int DEFAULT_PANE			= 1;
	private int _pane;
	
	private static final String REMOTE_PROFILE_NAME 	= "Remote Profile Name";
	private static final String HOME_AGENT_PLATFORM 	= "Platform Name";
	private static final String ADDRESSES			= "Location of published addresses";
	private static final String INTERNAL_ADDRESSES		= "Platform Transport Naming Services";
	private static final String EXTERNAL_ADDRESSES		= "External Transport Naming Services";
	private static final String INTERNAL_ADDRESS		= "Naming Service URL";
	private static final String EXTERNAL_ADDRESS		= "Naming Service URL";
	private static final String DATABASE_LOCATION		= "Database Location";
	private static final String DATABASE_TYPE		= "Database Type";
	private static final String EDIT_REMOTE_PLATFORMS	= "Edit Remote Platforms";
	private static final String LOCAL_ADDRESS_LOCATION	= "Publish Transport Addresses To";
	private static final String PROTOCOL_MAPPINGS		= "Protocol Mappings";
	
	private static final String ADD				= "Add";
	private static final String EDIT			= "Edit";
	private static final String DELETE			= "Delete";
	
	private static final String SERIALIZATION_DATABASE 	= "SerializationDatabase";
	private static final String MEMORY_DATABASE		= "MemoryDatabase";
	private static final String NO_DATABASE			= "NoDatabase";	
	
	private static String FIPA_OS_LOCATION 			= "FIPA-OS Installation";
	private static String FIPA_OS_VERSION	 		= "FIPA-OS Version";
	private static String PLATFORM_PROFILE_LOCATION		= "Location of platform.profile";
	private static String VM_LOCATION			= "Location of Java VM";
	private static String VM_JIT_SETTING			= "VM JIT/HotSpot Setting";
	private static String SHOW_DEBUG_OUTPUT			= "Show output for debugging";
	private static String USE_LOADER_GUI			= "Use AgentLoader GUI";
	private static String DEBUG_LEVEL_MEM			= "Memory debug level";
	private static String DEBUG_LEVEL_FILE			= "File debug level";
	
	private static String AMS_URL				= "AMS URL";
	private static String HAP_NAME				= "Home Agent Platform Name";
	private static String PROFILE_DIR			= "Profile Directory";
	
	private static String AGENT_LOADER_MAPPINGS		= "Agent Loader Mappings";	
	
	private static String MANAGE_THREAD_POOLING		= "Thread Pool Management";
	private static String MANAGE				= "Manage";
	
	private static JPanel _spacer;
	
	static
	{
		
		GridBagLayout 		spacer_gridbag 		= new GridBagLayout();
		GridBagConstraints 	spacer_constraints	= new GridBagConstraints();
		
		spacer_constraints.fill 	= GridBagConstraints.BOTH;
		spacer_constraints.ipadx 	= 25;
		spacer_constraints.ipady	= 10;
		
		_spacer = new JPanel();
		
		spacer_gridbag.setConstraints(_spacer, spacer_constraints);
		
	}
	
	//-----------------------------------------------------------------------
	// ACC Profile Editor variables
	//-----------------------------------------------------------------------
	private JPanel _jp_acc;
	
	// external transport address (multiple string)
	private JTextArea _jta_external_addresses;
	
	// Local address location (single string)
	private JTextField _jtf_local_address_location;
	
	// remote agent platform profiles
	private JButton _jb_add_remote_platform;
	private JButton _jb_edit_remote_platform;
	private JButton _jb_del_remote_platform;
	private Vector _remote_platform_profiles;
	private JList _jli_remote_profiles;
	
	// internal transport addresses
	private JButton _jb_acc_add_internal_transport;
	private JButton _jb_acc_edit_internal_transport;
	private JButton _jb_acc_del_internal_transport;
	private Vector _acc_internal_transport_addresses;
	private JList _jli_acc_internal_transports;
	
	// external transport addresses
	private JButton _jb_acc_add_external_transport;
	private JButton _jb_acc_edit_external_transport;
	private JButton _jb_acc_del_external_transport;
	private Vector _acc_external_transport_addresses;
	private JList _jli_acc_external_transports;
	
	// database profile (single profile) - type string + location string
	private JComboBox 	_jcb_acc_database_type;
	private JTextField 	_jtf_acc_database_location;
	
	//-----------------------------------------------------------------------
	// Platform Profile Editor variables
	//-----------------------------------------------------------------------
	
	private JPanel _jp_platform;
	
	// home agent platform name
	private JTextField _jtf_hap_name;
	
	// AMS url
	private JTextField _jtf_ams_url;
	
	// Profile directory
	private JTextField _jtf_profile_directory;
	
	private JPanel _jp_setup;
	
	private JPanel _jp_buttons;
	
	private JButton _jb_save;
	private JButton _jb_exit;
	
	//-----------------------------------------------------------------------
	// Setup Runtime Profile variables
	//-----------------------------------------------------------------------
	
	private static final String JIT_KEY1 = "Normal VM Operation";
	private static final String JIT_KEY2 = "JIT Disabled";
	private static final String JIT_KEY3 = "HotSpot Disabled";
	
	private static final String DEBUG_KEY1 = "Yes";
	private static final String DEBUG_KEY2 = "No";
	
	private static final String LOADER_GUI_KEY1 = "Yes";
	private static final String LOADER_GUI_KEY2 = "No";
	
	private static String DBG_LVL_KEY1	= "Highest";
	private static String DBG_LVL_KEY2	= "High";
	private static String DBG_LVL_KEY3	= "Medium";
	private static String DBG_LVL_KEY4	= "Low";
	private static String DBG_LVL_KEY5	= "Lowest";

	private JTextField 	_jtf_fipa_os_location;
	private JTextField 	_jtf_fipa_os_ver;
	private JTextField 	_jtf_jdk_location;
	
	private JTextField 	_jtf_platform_profile_location;
	
	// Debug level combo boxes
	private JComboBox 	_jcb_dbg_lvl_mem;
	private JComboBox 	_jcb_dbg_lvl_file;
	private TwoWayMapping	_dbg_lvl_mappings;
	
	
	private JComboBox 	_jcb_jit_option;
	
	private JComboBox 	_jcb_debug_option;
	
	private JComboBox 	_jcb_loader_gui_option;
	
	//-----------------------------------------------------------------------
	// Default Profile Editor variables
	//-----------------------------------------------------------------------
	private JPanel _jp_default;
	
	// internal transport addresses
	private JButton _jb_default_add_internal_transport;
	private JButton _jb_default_edit_internal_transport;
	private JButton _jb_default_del_internal_transport;
	
	private Vector _default_internal_transport_addresses;
	
	private JList _jli_default_internal_transports;
	
	// protocol mappings
	private JButton _jb_default_add_protocol_mapping;
	private JButton _jb_default_edit_protocol_mapping;
	private JButton _jb_default_del_protocol_mapping;
	
	private JButton _jb_default_manage_thread_pooling;	
	
	private Vector _default_protocol_mappings;
	
	private JList _jli_default_protocol_mappings;
	
	// database profile (single profile) - type string + location string
	private JComboBox 	_jcb_default_database_type;
	private JTextField 	_jtf_default_database_location;
	
	private FIPAOSConfigurator _owner;
	
	//-----------------------------------------------------------------------
	// Loader Profile Editor variables
	//-----------------------------------------------------------------------
	
	private JPanel _jp_loader;
	
	// loader editor
	private JButton _jb_loader_add_agent_mapping;
	private JButton _jb_loader_edit_agent_mapping;
	private JButton _jb_loader_del_agent_mapping;
	
	private Vector _loader_agent_mappings;
	
	private JList _jli_loader_agent_mappings;
	
	/*********************************************************
	 *
	 * FIPAOSConfiguratorGUI()
	 *
	 * Constructor
	 *
	 *********************************************************/
	public FIPAOSConfiguratorGUI(
	FIPAOSConfigurator owner,
	PlatformProfile platform_profile,
	ACCProfile acc_profile,
	AgentProfile default_profile,
	LoaderProfile loader_profile,
	RuntimeProfile runtime_profile)
	{
		
		System.out.println("GUI running");
		
		_owner = owner;
		
		_platform_profile = platform_profile;
		
		if (_platform_profile == null)
		{
			_platform_profile = new PlatformProfileImpl();
		}
		
		_acc_profile = acc_profile;
		
		if (_acc_profile == null)
		{
			_acc_profile = new ACCProfileImpl();
		}
		
		_default_profile = default_profile;
		
		if (_default_profile == null)
		{
			_default_profile = new AgentProfileImpl();
			_tm_profile = new TaskManagerProfileImpl();
		}
		
		_loader_profile = loader_profile;
		
		if (_loader_profile == null)
		{
			_loader_profile = new LoaderProfileImpl();
		}		
		
		_runtime_profile = runtime_profile;
		
		// Attempt to set the default L&F for the platform we're running on
		try
		{
			UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
		}
		catch( Throwable t )
		{
			DIAGNOSTICS.println( "Couldn't set the default Look & Feel for this OS", this, DIAGNOSTICS.LEVEL_3 );
		}
		
		setIconImage( new ImageIcon( ClassLoader.getSystemResource( "images/logo.gif" ) ).getImage() );
		setTitle(title);
		setResizable(true);
		
		addWindowListener(this);
		setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
		
		GridBagLayout main_gridbag 		= new GridBagLayout();
		GridBagConstraints main_constraints	= new GridBagConstraints();
		
		main_constraints.fill 			= GridBagConstraints.BOTH;
		main_constraints.gridwidth 		= GridBagConstraints.REMAINDER;
		main_constraints.anchor		 	= GridBagConstraints.NORTH;
		main_constraints.ipadx 			= 25;
		
		JPanel jp_main = new JPanel(main_gridbag);
		JTabbedPane jtp_main = new JTabbedPane();
		
		main_gridbag.setConstraints(jtp_main, main_constraints);
		
		_dbg_lvl_mappings = new TwoWayMapping();
		
		_dbg_lvl_mappings.put(DBG_LVL_KEY1, new Integer(1));
		_dbg_lvl_mappings.put(DBG_LVL_KEY2, new Integer(2));
		_dbg_lvl_mappings.put(DBG_LVL_KEY3, new Integer(3));
		_dbg_lvl_mappings.put(DBG_LVL_KEY4, new Integer(4));
		_dbg_lvl_mappings.put(DBG_LVL_KEY5, new Integer(5));
		
		// initialize member variables for remote platform list box
		_jli_remote_profiles 		= new JList();
		_remote_platform_profiles 	= new Vector();
		_jb_add_remote_platform 	= new JButton(ADD);
		_jb_edit_remote_platform 	= new JButton(EDIT);
		_jb_del_remote_platform 	= new JButton(DELETE);
		
		// initialize member variables for acc internal transports list box
		_jli_acc_internal_transports			= new JList();
		_acc_internal_transport_addresses		= new Vector();
		_jb_acc_add_internal_transport			= new JButton(ADD);
		_jb_acc_edit_internal_transport			= new JButton(EDIT);
		_jb_acc_del_internal_transport			= new JButton(DELETE);
		
		// initialize member variables for default internal transports list box
		_jli_default_internal_transports		= new JList();
		_default_internal_transport_addresses		= new Vector();
		_jb_default_add_internal_transport		= new JButton(ADD);
		_jb_default_edit_internal_transport		= new JButton(EDIT);
		_jb_default_del_internal_transport		= new JButton(DELETE);
		
		// initialize member variables for default profile protocol mappings
		_jli_default_protocol_mappings			= new JList();
		_default_protocol_mappings			= new Vector();
		_jb_default_add_protocol_mapping		= new JButton(ADD);
		_jb_default_edit_protocol_mapping		= new JButton(EDIT);
		_jb_default_del_protocol_mapping		= new JButton(DELETE);
		
		// initialize member variables for acc external transports list box
		_jli_acc_external_transports			= new JList();
		_acc_external_transport_addresses		= new Vector();
		_jb_acc_add_external_transport			= new JButton(ADD);
		_jb_acc_edit_external_transport			= new JButton(EDIT);
		_jb_acc_del_external_transport			= new JButton(DELETE);
		
		// initialize member variables for default profile protocol mappings
		_jli_loader_agent_mappings			= new JList();
		_loader_agent_mappings				= new Vector();
		_jb_loader_add_agent_mapping			= new JButton(ADD);
		_jb_loader_edit_agent_mapping			= new JButton(EDIT);
		_jb_loader_del_agent_mapping			= new JButton(DELETE);		
		
		// make the panels for the different profiles
		this.makeACCPane();
		this.makePlatformPane();
		this.makeSetupPane();
		this.makeDefaultPane();
		this.makeLoaderPane();
		this.makeButtonsPane();
		
		main_gridbag.setConstraints(_jp_buttons, main_constraints);
		
		jtp_main.add(_jp_acc, ACC_PROFILE);
		
		jtp_main.add(_jp_platform, PLATFORM_PROFILE);
		
		jtp_main.add(_jp_setup, RUNTIME_PROFILE);
		
		jtp_main.add(_jp_default, DEFAULT_PROFILE);
		
		jtp_main.add(_jp_loader, LOADER_PROFILE);
		
		jp_main.add(jtp_main);
		jp_main.add(_jp_buttons);
		
		getContentPane().add(jp_main);
		
		this.readDataIntoGUI();
		
		this.updateLists();
		
		this.centre();
		
		this.setVisible(true);
		
	} // end FIPAOSConfiguratorGUI()
	
	/*********************************************************
	 *
	 * makeACCPane()
	 *
	 * Creates the JPanel for the ACC profile editor pane
	 *
	 *********************************************************/
	private void makeACCPane()
	{
		
		GridBagLayout 		acc_gridbag 	= new GridBagLayout();
		GridBagConstraints 	acc_constraints	= new GridBagConstraints();
		
		acc_constraints.fill 			= GridBagConstraints.BOTH;
		acc_constraints.gridwidth 		= GridBagConstraints.REMAINDER;
		acc_constraints.weightx			= 1.0;
		acc_constraints.ipadx 			= 25;
		
		_jp_acc = new JPanel(acc_gridbag);
		
		//--------------------------------------------------------------
		// Set up internal addresses editor
		//--------------------------------------------------------------
		
		JLabel jl_int_addr	= new JLabel(INTERNAL_ADDRESSES);
		acc_gridbag.setConstraints(jl_int_addr, acc_constraints);
		
		_jp_acc.add(jl_int_addr);
		
		JPanel jp_internal_transports = makeListEditPanel(_jli_acc_internal_transports,
		_jb_acc_add_internal_transport,
		_jb_acc_edit_internal_transport,
		_jb_acc_del_internal_transport);
		
		acc_gridbag.setConstraints(jp_internal_transports, acc_constraints);
		
		_jp_acc.add(jp_internal_transports);
		
		//--------------------------------------------------------------
		// Set up external addresses editor
		//--------------------------------------------------------------
		
		JLabel jl_ext_addr	= new JLabel(EXTERNAL_ADDRESSES);
		acc_gridbag.setConstraints(jl_ext_addr, acc_constraints);
		
		_jp_acc.add(jl_ext_addr);
		
		JPanel jp_external_transports = makeListEditPanel(_jli_acc_external_transports,
		_jb_acc_add_external_transport,
		_jb_acc_edit_external_transport,
		_jb_acc_del_external_transport);
		
		acc_gridbag.setConstraints(jp_external_transports, acc_constraints);
		
		_jp_acc.add(jp_external_transports);
		
		//--------------------------------------------------------------
		// Set up local address editor
		//--------------------------------------------------------------
		
		JLabel jl_addr_loc	= new JLabel(LOCAL_ADDRESS_LOCATION);
		acc_gridbag.setConstraints(jl_addr_loc, acc_constraints);
		
		_jp_acc.add(jl_addr_loc);
		
		_jtf_local_address_location = new JTextField(32);
		acc_gridbag.setConstraints(_jtf_local_address_location, acc_constraints);
		
		_jp_acc.add(_jtf_local_address_location);
		
		//--------------------------------------------------------------
		// Set up remote platforms editor
		//--------------------------------------------------------------
		
		JLabel jl_edit_remotes = new JLabel(EDIT_REMOTE_PLATFORMS);
		acc_gridbag.setConstraints(jl_edit_remotes, acc_constraints);
		
		_jp_acc.add(jl_edit_remotes);
		
		JPanel jp_remote_platform_panel = makeListEditPanel(_jli_remote_profiles,
		_jb_add_remote_platform,
		_jb_edit_remote_platform,
		_jb_del_remote_platform);
		
		acc_gridbag.setConstraints(jp_remote_platform_panel, acc_constraints);
		
		_jp_acc.add(jp_remote_platform_panel);
		
		//--------------------------------------------------------------
		// Set up databse type editor
		//--------------------------------------------------------------
		
		JLabel jl_db_type	= new JLabel(DATABASE_TYPE);
		acc_gridbag.setConstraints(jl_db_type, acc_constraints);
		
		_jp_acc.add(jl_db_type);
		
		_jcb_acc_database_type = new JComboBox();
		
		_jcb_acc_database_type.addItem(SERIALIZATION_DATABASE);
		_jcb_acc_database_type.addItem(MEMORY_DATABASE);
		
		acc_gridbag.setConstraints(_jcb_acc_database_type, acc_constraints);
		
		_jp_acc.add(_jcb_acc_database_type);
		
		//--------------------------------------------------------------
		// Set up database location editor
		//--------------------------------------------------------------
		
		JLabel jl_db_loc	= new JLabel(DATABASE_LOCATION);
		acc_gridbag.setConstraints(jl_db_loc, acc_constraints);
		
		_jtf_acc_database_location = new JTextField(32);
		acc_gridbag.setConstraints(_jtf_acc_database_location, acc_constraints);
		
		_jp_acc.add(jl_db_loc);
		_jp_acc.add(_jtf_acc_database_location);
		
	} // end makeACCPane()
	
	/*********************************************************
	 *
	 * makeListEditPanel()
	 *
	 * @param list. The JList to create to contain the editable items
	 * @param add. The JButton to create for adding a new item
	 * @param edit. The JButton to create for editing an existing item
	 * @param delete. JButton to create for deleting an existing item
	 *
	 * @return JPanel. The edit panel with the buttons
	 *
	 *********************************************************/
	private JPanel makeListEditPanel(JList list,
	JButton add,
	JButton edit,
	JButton delete)
	{
		
		GridBagLayout 		panel_gridbag 		= new GridBagLayout();
		GridBagConstraints 	panel_constraints	= new GridBagConstraints();
		
		panel_constraints.fill 				= GridBagConstraints.BOTH;
		panel_constraints.ipadx 			= 25;
		
		JPanel panel = new JPanel(panel_gridbag);
		
		JScrollPane scrollPane = new JScrollPane(list);
		scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		list.setFixedCellWidth(300);
		list.setVisibleRowCount(3);
		
		panel_constraints.gridheight	= 3;
		
		panel_gridbag.setConstraints(scrollPane, panel_constraints);
		
		panel_constraints.gridwidth 	= GridBagConstraints.REMAINDER;
		panel_constraints.gridheight 	= 1;
		
		add.addActionListener(this);
		panel_gridbag.setConstraints(add, panel_constraints);
		
		edit.addActionListener(this);
		panel_gridbag.setConstraints(edit, panel_constraints);
		
		delete.addActionListener(this);
		panel_gridbag.setConstraints(delete, panel_constraints);
		
		panel.add(scrollPane);
		panel.add(add);
		panel.add(edit);
		panel.add(delete);
		
		return panel;
		
	} // end makeListEditPanel()
	
	/*********************************************************
	 *
	 * makePlatformPane()
	 *
	 * Creates the JPanel for the Platform profile editor pane
	 *
	 *********************************************************/
	private void makePlatformPane()
	{
		
		GridBagLayout 		platform_gridbag 	= new GridBagLayout();
		GridBagConstraints 	platform_constraints	= new GridBagConstraints();
		
		platform_constraints.fill 			= GridBagConstraints.BOTH;
		platform_constraints.gridwidth 			= GridBagConstraints.REMAINDER;
		platform_constraints.ipadx 			= 25;
		platform_constraints.weightx			= 1.0;
		
		_jp_platform = new JPanel(new FlowLayout());
		
		JPanel jp_sub_platform = new JPanel(platform_gridbag);
		
		//--------------------------------------------------------------
		// Set up HAP edit field
		//--------------------------------------------------------------
		
		JLabel jl_hap_name	= new JLabel(HAP_NAME);
		platform_gridbag.setConstraints(jl_hap_name, platform_constraints);
		jp_sub_platform.add(jl_hap_name);
		
		_jtf_hap_name = new JTextField(32);
		platform_gridbag.setConstraints(_jtf_hap_name, platform_constraints);
		jp_sub_platform.add(_jtf_hap_name);
		
		//--------------------------------------------------------------
		// Set up AMS address edit field
		//--------------------------------------------------------------
		
		JLabel jl_ams_url	= new JLabel(AMS_URL);
		platform_gridbag.setConstraints(jl_ams_url, platform_constraints);
		jp_sub_platform.add(jl_ams_url);
		
		_jtf_ams_url = new JTextField(32);
		platform_gridbag.setConstraints(_jtf_ams_url, platform_constraints);
		jp_sub_platform.add(_jtf_ams_url);
		
		//--------------------------------------------------------------
		// Set up Profile directory edit field
		//--------------------------------------------------------------
		
		JLabel jl_profile_dir	= new JLabel(PROFILE_DIR);
		platform_gridbag.setConstraints(jl_profile_dir, platform_constraints);
		jp_sub_platform.add(jl_profile_dir);
		
		_jtf_profile_directory = new JTextField(32);
		platform_gridbag.setConstraints(_jtf_profile_directory, platform_constraints);
		jp_sub_platform.add(_jtf_profile_directory);
		
		// now add the components JPanel to the main JPanel
		_jp_platform.add(jp_sub_platform);
		
	} // end makePlatformPane()
	
	/*********************************************************
	 *
	 * makeSetupPane()
	 *
	 * Creates the JPanel for the setup runtime profile pane
	 *
	 *********************************************************/
	private void makeSetupPane()
	{
		
		GridBagLayout 		setup_gridbag 		= new GridBagLayout();
		GridBagConstraints 	setup_constraints	= new GridBagConstraints();
		
		setup_constraints.fill 		= GridBagConstraints.BOTH;
		setup_constraints.ipadx 	= 25;
		setup_constraints.weightx	= 1.0;
		
		_jp_setup = new JPanel(new FlowLayout());
		
		JPanel jp_sub_setup = new JPanel(setup_gridbag);
		
		//--------------------------------------------------------------
		// Set up FIPAOS text field
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_fipa_os_loc = new JLabel(FIPA_OS_LOCATION);
		setup_gridbag.setConstraints(jl_fipa_os_loc, setup_constraints);
		jp_sub_setup.add(jl_fipa_os_loc);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jtf_fipa_os_location = new JTextField();
		setup_gridbag.setConstraints(_jtf_fipa_os_location, setup_constraints);
		jp_sub_setup.add(_jtf_fipa_os_location);
		
		//--------------------------------------------------------------
		// Set up FIPA_OS_VER text field
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_fipa_os_ver = new JLabel(FIPA_OS_VERSION);
		setup_gridbag.setConstraints(jl_fipa_os_ver, setup_constraints);
		jp_sub_setup.add(jl_fipa_os_ver);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jtf_fipa_os_ver = new JTextField();
		setup_gridbag.setConstraints(_jtf_fipa_os_ver, setup_constraints);
		jp_sub_setup.add(_jtf_fipa_os_ver);
		
		//--------------------------------------------------------------
		// Set up Platform Profile text field
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_platform_profile = new JLabel(PLATFORM_PROFILE_LOCATION);
		setup_gridbag.setConstraints(jl_platform_profile, setup_constraints);
		jp_sub_setup.add(jl_platform_profile);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jtf_platform_profile_location = new JTextField();
		setup_gridbag.setConstraints(_jtf_platform_profile_location, setup_constraints);
		jp_sub_setup.add(_jtf_platform_profile_location);
		
		//--------------------------------------------------------------
		// Set up JDK Version text field
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_jdk_location = new JLabel(VM_LOCATION);
		setup_gridbag.setConstraints(jl_jdk_location, setup_constraints);
		jp_sub_setup.add(jl_jdk_location);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jtf_jdk_location = new JTextField();
		setup_gridbag.setConstraints(_jtf_jdk_location, setup_constraints);
		jp_sub_setup.add(_jtf_jdk_location);
		
		//--------------------------------------------------------------
		// Set up JIT Selector
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_jit = new JLabel(VM_JIT_SETTING);
		setup_gridbag.setConstraints(jl_jit, setup_constraints);
		jp_sub_setup.add(jl_jit);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jcb_jit_option = new JComboBox();
		
		_jcb_jit_option.addItem(JIT_KEY1);
		_jcb_jit_option.addItem(JIT_KEY2);
		_jcb_jit_option.addItem(JIT_KEY3);
		
		setup_gridbag.setConstraints(_jcb_jit_option, setup_constraints);
		jp_sub_setup.add(_jcb_jit_option);
		
		//--------------------------------------------------------------
		// Set up Debug Selector
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_debug = new JLabel(SHOW_DEBUG_OUTPUT);
		setup_gridbag.setConstraints(jl_debug, setup_constraints);
		jp_sub_setup.add(jl_debug);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jcb_debug_option = new JComboBox();
		
		_jcb_debug_option.addItem(DEBUG_KEY1);
		_jcb_debug_option.addItem(DEBUG_KEY2);
		
		setup_gridbag.setConstraints(_jcb_debug_option, setup_constraints);
		jp_sub_setup.add(_jcb_debug_option);
		
		//--------------------------------------------------------------
		// Set up Loader GUI selector
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_loader_gui = new JLabel(USE_LOADER_GUI);
		setup_gridbag.setConstraints(jl_loader_gui, setup_constraints);
		jp_sub_setup.add(jl_loader_gui);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jcb_loader_gui_option = new JComboBox();
		
		_jcb_loader_gui_option.addItem(LOADER_GUI_KEY1);
		_jcb_loader_gui_option.addItem(LOADER_GUI_KEY2);
		
		setup_gridbag.setConstraints(_jcb_loader_gui_option, setup_constraints);
		jp_sub_setup.add(_jcb_loader_gui_option);
		
		//--------------------------------------------------------------
		// Set up Debug Memory Level
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_debug_lvl_mem = new JLabel(DEBUG_LEVEL_MEM);
		setup_gridbag.setConstraints(jl_debug_lvl_mem, setup_constraints);
		jp_sub_setup.add(jl_debug_lvl_mem);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jcb_dbg_lvl_mem = new JComboBox();
		
		_jcb_dbg_lvl_mem.addItem(DBG_LVL_KEY1);
		_jcb_dbg_lvl_mem.addItem(DBG_LVL_KEY2);
		_jcb_dbg_lvl_mem.addItem(DBG_LVL_KEY3);
		_jcb_dbg_lvl_mem.addItem(DBG_LVL_KEY4);
		_jcb_dbg_lvl_mem.addItem(DBG_LVL_KEY5);
		
		
		setup_gridbag.setConstraints(_jcb_dbg_lvl_mem, setup_constraints);
		jp_sub_setup.add(_jcb_dbg_lvl_mem);
		
		//--------------------------------------------------------------
		// Set up Debug File Level
		//--------------------------------------------------------------
		
		setup_constraints.gridwidth = GridBagConstraints.RELATIVE;
		
		JLabel jl_debug_lvl_file = new JLabel(DEBUG_LEVEL_FILE);
		setup_gridbag.setConstraints(jl_debug_lvl_file, setup_constraints);
		jp_sub_setup.add(jl_debug_lvl_file);
		
		setup_constraints.gridwidth = GridBagConstraints.REMAINDER;
		
		_jcb_dbg_lvl_file = new JComboBox();
		
		_jcb_dbg_lvl_file.addItem(DBG_LVL_KEY1);
		_jcb_dbg_lvl_file.addItem(DBG_LVL_KEY2);
		_jcb_dbg_lvl_file.addItem(DBG_LVL_KEY3);
		_jcb_dbg_lvl_file.addItem(DBG_LVL_KEY4);
		_jcb_dbg_lvl_file.addItem(DBG_LVL_KEY5);
		
		setup_gridbag.setConstraints(_jcb_dbg_lvl_file, setup_constraints);
		jp_sub_setup.add(_jcb_dbg_lvl_file);
		
		//--------------------------------------------------------------
		// Now add the components JPanel to the main JPanel
		//--------------------------------------------------------------
		
		_jp_setup.add(jp_sub_setup);
		
	} // end makeSetupPane()
	
  /*********************************************************
   *
   * makeDefaultPane()
   *
   * Creates the JPanel for the default agent profile editor pane
   *
   *********************************************************/
	private void makeDefaultPane()
	{
		
		GridBagLayout 		default_gridbag 	= new GridBagLayout();
		GridBagConstraints 	default_constraints	= new GridBagConstraints();
		
		default_constraints.fill 	= GridBagConstraints.BOTH;
		default_constraints.gridwidth 	= GridBagConstraints.REMAINDER;
		default_constraints.weightx	= 1.0;
		default_constraints.ipadx 	= 25;	
		
		_jp_default = new JPanel(new FlowLayout());
		
		JPanel jp_sub_default = new JPanel(default_gridbag);
		
		//--------------------------------------------------------------
		// Set up internal addresses editor
		//--------------------------------------------------------------
		
		JLabel jl_int_addr	= new JLabel(INTERNAL_ADDRESSES);
		default_gridbag.setConstraints(jl_int_addr, default_constraints);
		
		jp_sub_default.add(jl_int_addr);
		
		JPanel jp_internal_transports = makeListEditPanel(_jli_default_internal_transports,
		_jb_default_add_internal_transport,
		_jb_default_edit_internal_transport,
		_jb_default_del_internal_transport);
		
		default_gridbag.setConstraints(jp_internal_transports, default_constraints);
		
		jp_sub_default.add(jp_internal_transports);
		
		//--------------------------------------------------------------
		// Set up protocol mappings editor
		//--------------------------------------------------------------
		
		JLabel jl_protocol_mappings = new JLabel(PROTOCOL_MAPPINGS);
		default_gridbag.setConstraints(jl_protocol_mappings, default_constraints);
		
		jp_sub_default.add(jl_protocol_mappings);
		
		JPanel jp_protocol_mappings = makeListEditPanel(_jli_default_protocol_mappings,
		_jb_default_add_protocol_mapping,
		_jb_default_edit_protocol_mapping,
		_jb_default_del_protocol_mapping);
		
		default_gridbag.setConstraints(jp_protocol_mappings, default_constraints);
		
		jp_sub_default.add(jp_protocol_mappings);
		
		//--------------------------------------------------------------
		// Set up databse type editor
		//--------------------------------------------------------------
		
		JLabel jl_db_type	= new JLabel(DATABASE_TYPE);
		default_gridbag.setConstraints(jl_db_type, default_constraints);
		
		jp_sub_default.add(jl_db_type);
		
		_jcb_default_database_type = new JComboBox();
		
		_jcb_default_database_type.addItem(SERIALIZATION_DATABASE);
		_jcb_default_database_type.addItem(MEMORY_DATABASE);
		_jcb_default_database_type.addItem(NO_DATABASE);
		
		default_gridbag.setConstraints(_jcb_default_database_type, default_constraints);
		
		jp_sub_default.add(_jcb_default_database_type);
		
		//--------------------------------------------------------------
		// Set up database location editor
		//--------------------------------------------------------------
		
		JLabel jl_db_loc	= new JLabel(DATABASE_LOCATION);
		default_gridbag.setConstraints(jl_db_loc, default_constraints);
		
		_jtf_default_database_location = new JTextField(32);
		default_gridbag.setConstraints(_jtf_default_database_location, default_constraints);	
		
		jp_sub_default.add(jl_db_loc);
		jp_sub_default.add(_jtf_default_database_location);
		
		//--------------------------------------------------------------
		// Thread Pooling management
		//--------------------------------------------------------------		

		JLabel jl_thread_pooling = new JLabel(MANAGE_THREAD_POOLING);
		default_gridbag.setConstraints(jl_thread_pooling, default_constraints);		
		
		_jb_default_manage_thread_pooling = new JButton(MANAGE);
		
		_jb_default_manage_thread_pooling.addActionListener(this);
		
		default_gridbag.setConstraints(_jb_default_manage_thread_pooling, default_constraints);		
		
		jp_sub_default.add(jl_thread_pooling);
		jp_sub_default.add(_jb_default_manage_thread_pooling);
		
		// now add the components JPanel to the main JPanel
		_jp_default.add(jp_sub_default);
		
	} // end makeDefaultPane()
	
  /*********************************************************
   *
   * makeLoaderPane()
   *
   * Creates the JPanel for the loader agent profile editor pane
   *
   *********************************************************/
	private void makeLoaderPane()
	{
		
		GridBagLayout 		loader_gridbag 	= new GridBagLayout();
		GridBagConstraints 	loader_constraints	= new GridBagConstraints();
		
		loader_constraints.fill 	= GridBagConstraints.BOTH;
		loader_constraints.gridwidth 	= GridBagConstraints.REMAINDER;
		loader_constraints.weightx	= 1.0;
		loader_constraints.ipadx 	= 25;
		
		_jp_loader= new JPanel(new FlowLayout());
		
		JPanel jp_sub_loader = new JPanel(loader_gridbag);
		
		//--------------------------------------------------------------
		// Set up loader mappings editor
		//--------------------------------------------------------------
		
		JLabel jl_loader_mappings = new JLabel(AGENT_LOADER_MAPPINGS);
		loader_gridbag.setConstraints(jl_loader_mappings, loader_constraints);
		
		jp_sub_loader.add(jl_loader_mappings);
		
		JPanel jp_loader_mappings = makeListEditPanel(_jli_loader_agent_mappings,
		_jb_loader_add_agent_mapping,
		_jb_loader_edit_agent_mapping,
		_jb_loader_del_agent_mapping);
		
		loader_gridbag.setConstraints(jp_loader_mappings, loader_constraints);
		
		jp_sub_loader.add(jp_loader_mappings);
		
		// now add the components JPanel to the main JPanel
		_jp_loader.add(jp_sub_loader);
		
	} // end makeLoaderPane()	
	
	/*********************************************************
	 *
	 * makeButtonsPane()
	 *
	 * Creates a JPanel for the save and exit buttons on the gui
	 *
	 *********************************************************/
	private void makeButtonsPane()
	{
		
		_jp_buttons = new JPanel(new FlowLayout(FlowLayout.CENTER));
		
		_jb_save = new JButton("Save");
		_jb_save.addActionListener(this);
		
		_jb_exit = new JButton("Exit");
		_jb_exit.addActionListener(this);
		
		_jp_buttons.add(_jb_save);
		_jp_buttons.add(_jb_exit);
		
	}
	
	/*********************************************************
	 *
	 * readDataIntoGUI
	 *
	 *********************************************************/
	private void readDataIntoGUI()
	{
		
		readACCData();
		readPlatformData();
		readRuntimeData();
		readDefaultData();
		readLoaderData();
		
	} // end readDataIntoGUI()
	
	private void readACCData()
	{
		_acc_external_transport_addresses = (Vector)_acc_profile.getExternalAddresss();
		
		if (_acc_external_transport_addresses == null)
		{
			_acc_external_transport_addresses = new Vector();
		}
		
		_acc_internal_transport_addresses = (Vector)_acc_profile.getInternalAddresss();
		
		if (_acc_internal_transport_addresses == null)
		{
			_acc_internal_transport_addresses = new Vector();
		}
		
		_jtf_local_address_location.setText(_acc_profile.getLocalAddressesLocation());
		
		DatabaseProfile acc_db = _acc_profile.getDatabaseProfile();
		
		if (acc_db == null)
		{
			acc_db = new DatabaseProfileImpl();
		}
		
		String db_type = acc_db.getDatabaseType();
		
		if (db_type != null)
		{
			
			if (db_type.equals(SERIALIZATION_DATABASE))
			{
				_jcb_acc_database_type.setSelectedItem(SERIALIZATION_DATABASE);
			}
			else
			{
				_jcb_acc_database_type.setSelectedItem(MEMORY_DATABASE);
			}
			
			_jtf_acc_database_location.setText(acc_db.getDatabaseLocation());
			
		}
		else
		{
			_jcb_acc_database_type.setSelectedItem(MEMORY_DATABASE);
		}
		
		_remote_platform_profiles = (Vector)_acc_profile.getRemoteAgentPlatformProfiles();
		
		if (_remote_platform_profiles == null)
		{
			_remote_platform_profiles = new Vector();
		}
		
	} // end readACCData()
	
	private void readPlatformData()
	{
		
		_jtf_hap_name.setText(_platform_profile.getHAPName());
		
		_jtf_ams_url.setText(_platform_profile.getAMSAddress());
		
		_jtf_profile_directory.setText(_platform_profile.getProfileDirectory());
		
	} // end readPlatformData()
	
	private void readRuntimeData()
	{
		
		_jtf_fipa_os_location.setText(_runtime_profile.getFipaosPath());
		_jtf_fipa_os_ver.setText(_runtime_profile.getFipaosVer());
		_jtf_jdk_location.setText(_runtime_profile.getJdkPath());
		_jtf_platform_profile_location.setText(_runtime_profile.getPlatformPath());
		
		// select debug combo box option
		if (_runtime_profile.getDebugger() == true)
		{
			_jcb_debug_option.setSelectedItem(DEBUG_KEY1);
		}
		else
		{
			_jcb_debug_option.setSelectedItem(DEBUG_KEY2);
		}
		
		// select loader gui combo box option
		if (_runtime_profile.getHasGui() == true)
		{
			_jcb_loader_gui_option.setSelectedItem(LOADER_GUI_KEY1);
		}
		else
		{
			_jcb_loader_gui_option.setSelectedItem(LOADER_GUI_KEY2);
		}
		
		String key;
		String value;
		
		// select jit combo box option
        if (_runtime_profile.getNoJit() == ScriptVars.JIT_NORMAL)
			_jcb_jit_option.setSelectedItem(JIT_KEY1);
		else if (_runtime_profile.getNoJit() ==  ScriptVars.JIT_NOJIT)
			_jcb_jit_option.setSelectedItem(JIT_KEY2);
		else if (_runtime_profile.getNoJit() == ScriptVars.JIT_NOHOTSPOT)
			_jcb_jit_option.setSelectedItem(JIT_KEY3);
		
		// select memory debug level combo box option
		_jcb_dbg_lvl_mem.setSelectedItem((String)_dbg_lvl_mappings.get(_runtime_profile.getDebugMemLvl()));
		
		// select file debug level combo box option
		_jcb_dbg_lvl_file.setSelectedItem((String)_dbg_lvl_mappings.get(_runtime_profile.getDebugFileLvl()));
		
	} // readRuntimeData()
	
	private void readDefaultData()
	{
		
		// get internal transport addresses
		_default_internal_transport_addresses = (Vector)_default_profile.getInternalAddresss();
		
		if (_default_internal_transport_addresses == null)
		{
			_default_internal_transport_addresses = new Vector();
		}
		
		// get protocol mappings
		ProtocolProfile protocol_profile = _default_profile.getProtocolProfile();
		
		if (protocol_profile == null)
		{
			protocol_profile = new ProtocolProfileImpl();
		}
		
		_default_protocol_mappings = (Vector)protocol_profile.getProtocolMappings();
		
		if (_default_protocol_mappings == null)
		{
			_default_protocol_mappings = new Vector();
		}
		
		// get database profile
		
		DatabaseProfile default_db = _default_profile.getDatabaseProfile();
		
		if (default_db == null)
		{
			default_db = new DatabaseProfileImpl();
		}
		
		String default_db_type = default_db.getDatabaseType();
		
		if (default_db_type != null)
		{
			
			if (default_db_type.equals(SERIALIZATION_DATABASE))
			{
				_jcb_default_database_type.setSelectedItem(SERIALIZATION_DATABASE);
			}
			else if (default_db_type.equals(MEMORY_DATABASE))
			{
				_jcb_default_database_type.setSelectedItem(MEMORY_DATABASE);
			}
			else
			{
				_jcb_default_database_type.setSelectedItem(NO_DATABASE);
			}			
			
			_jtf_default_database_location.setText(default_db.getDatabaseLocation());
			
		}
		else
		{
			_jcb_default_database_type.setSelectedItem(NO_DATABASE);
		}
		
		if (_default_profile.getTaskManagerProfile() != null)
		{
			_tm_profile = _default_profile.getTaskManagerProfile();
		}
		else
		{
			_tm_profile = new TaskManagerProfileImpl();
		}
		
		
	} // end readDefaultData()
	
	private void readLoaderData()
	{
		
		_loader_agent_mappings = (Vector)_loader_profile.getAgentDescriptions();
		
		if (_loader_agent_mappings == null)
		{
			_loader_agent_mappings = new Vector();
		}
		
	} // end readLoaderData()		
	
	/*********************************************************
	 *
	 * updateList()
	 *
	 * @param hash. The Hashtable containing the new values
	 * @param list. The JList to update on screen
	 *
	 *********************************************************/
	private void updateList(java.util.List list, JList jlist)
	{
		if (list != null)
		{
			Iterator iter = list.iterator();
			
			Vector vect = new Vector();
			
			Object obj;
			
			while (iter.hasNext())
			{
				obj = iter.next();
				
				if (obj instanceof InternalAddressImpl)
				{
					vect.add(((InternalAddressImpl)obj).getAddress());					
				}
				else if (obj instanceof ExternalAddressImpl)
				{
					vect.add(((ExternalAddressImpl)obj).getAddress());					
				}
				else if (obj instanceof RemoteAgentPlatformProfileImpl)
				{
					vect.add(((RemoteAgentPlatformProfileImpl)obj).getHAPName());					
				}
				else if (obj instanceof ProtocolMappingImpl)
				{
					vect.add(((ProtocolMappingImpl)obj).getProtocolName());					
				}
				else if (obj instanceof AgentDescription)
				{
					vect.add(((AgentDescription)obj).getAgentName());					
				}				
			}
						
			jlist.setListData(vect.toArray());
		}
		
	} // end updateList()
	
/***********************************************************************
 *
 * actionPerformed()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void actionPerformed(ActionEvent event)
	{
		
		Object source = event.getSource();
		
		if (source instanceof JButton)
		{
			this.handleButtonPressed(source);
		}
		
		updateLists();
		
	} // end actionPerformed()
	
/***********************************************************************
 *
 * windowOpened()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void windowOpened(WindowEvent event)
	{
		
	} // end windowOpened()
	
/***********************************************************************
 *
 * windowClosing()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void windowClosing(WindowEvent event)
	{
		
		this.confirmExit();
		
	} // end windowClosing()
	
/***********************************************************************
 *
 * windowClosed()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void windowClosed(WindowEvent event)
	{
		
	} // end windowClosed()
	
/***********************************************************************
 *
 * windowIconified()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void windowIconified(WindowEvent event)
	{
		
	} // end windowIconified()
	
/***********************************************************************
 *
 * windowDeiconified()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void windowDeiconified(WindowEvent event)
	{
		
	} // end windowDeiconified()
	
/***********************************************************************
 *
 * windowActivated()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void windowActivated(WindowEvent event)
	{
		
	} // end windowActivated()
	
/***********************************************************************
 *
 * windowDeactivated()
 *
 * @param event. The event that was performed
 *
 ***********************************************************************/
	public void windowDeactivated(WindowEvent event)
	{
		
	} // end windowDeactivated()
	
/***********************************************************************
 *
 * handleButtonPressed()
 *
 * @param pressed. The button that was pressed
 *
 ***********************************************************************/
	public void handleButtonPressed(Object pressed)
	{
		
		if (pressed == _jb_save)
		{
			this.saveNewValues();
		}
		else if (pressed == _jb_exit)
		{
			this.confirmExit();
		}
		else if (pressed == _jb_add_remote_platform)
		{
			this.addRemotePlatformProfile();
		}
		else if (pressed == _jb_edit_remote_platform)
		{
			this.editRemotePlatformProfile();
		}
		else if (pressed == _jb_del_remote_platform)
		{
			this.delListItem(_remote_platform_profiles, _jli_remote_profiles);
		}
		else if (pressed == _jb_acc_add_internal_transport)
		{
			_pane = ACC_PANE;
			
			this.addInternalTransport();
		}
		else if (pressed == _jb_acc_edit_internal_transport)
		{
			_pane = ACC_PANE;
			
			this.editInternalTransport();
		}
		else if (pressed == _jb_acc_del_internal_transport)
		{
			this.delListItem(_acc_internal_transport_addresses, _jli_acc_internal_transports);
		}
		else if (pressed == _jb_default_add_internal_transport)
		{
			_pane = DEFAULT_PANE;
			
			this.addInternalTransport();
		}
		else if (pressed == _jb_default_edit_internal_transport)
		{
			_pane = DEFAULT_PANE;
			
			this.editInternalTransport();
		}
		else if (pressed == _jb_default_del_internal_transport)
		{
			this.delListItem(_default_internal_transport_addresses, _jli_default_internal_transports);
		}
		else if (pressed == _jb_default_add_protocol_mapping)
		{
			this.addProtocolMapping();
		}
		else if (pressed == _jb_default_edit_protocol_mapping)
		{
			this.editProtocolMapping();
		}
		else if (pressed == _jb_default_del_protocol_mapping)
		{
			this.delListItem(_default_protocol_mappings, _jli_default_protocol_mappings);
		}
		else if (pressed == _jb_acc_add_external_transport)
		{
			this.addExternalTransport();
		}
		else if (pressed == _jb_acc_edit_external_transport)
		{
			this.editExternalTransport();
		}
		else if (pressed == _jb_acc_del_external_transport)
		{
			this.delListItem(_acc_external_transport_addresses, _jli_acc_external_transports);
		}
		else if (pressed == _jb_loader_add_agent_mapping)
		{
			this.addLoaderAgentDescription();
		}
		else if (pressed == _jb_loader_edit_agent_mapping)
		{
			this.editLoaderAgentDescription();
		}
		else if (pressed == _jb_loader_del_agent_mapping)
		{
			this.delListItem(_loader_agent_mappings, _jli_loader_agent_mappings);
		}
		else if (pressed == _jb_default_manage_thread_pooling)
		{
			this.editThreadPooling();
		}			
		
	} // end handleButtonPressed()
	
/***********************************************************************
 *
 * editThreadPooling()
 *
 ***********************************************************************/
	private void editThreadPooling()
	{
		//<taskManagerProfile useGlobalThreads="true" localThreads="8"/>
		
		DataBindingEditor editor = new DataBindingEditor(this, _tm_profile, EDITING);
		
	} // end editThreadPooling()	
	
/***********************************************************************
 *
 * addRemotePlatformProfile()
 *
 * Creates a new remote platform description object when the add button is
 * pressed on the remote platforms list.
 *
 ***********************************************************************/
	private void addRemotePlatformProfile()
	{
		
		RemoteAgentPlatformProfile new_profile = new RemoteAgentPlatformProfileImpl();
		
		_remote_platform_profiles.add(new_profile);
		
		DataBindingEditor editor = new DataBindingEditor(this, new_profile, ADDING);
		
		updateLists();
		
	} // end addRemotePlatformProfile()
	
/***********************************************************************
 *
 * editRemotePlatformProfile()
 *
 * Edits one of the remote platform descriptions  when the edit button is
 * pressed on the remote platforms list.
 *
 ***********************************************************************/
	private void editRemotePlatformProfile()
	{
		
		int index = _jli_remote_profiles.getSelectedIndex();
		
		if (index != -1)
		{
			
			RemoteAgentPlatformProfile to_edit = (RemoteAgentPlatformProfile)(_remote_platform_profiles.get(index));
			
			if (to_edit != null)
			{
				
				DataBindingEditor editor = new DataBindingEditor(this, to_edit, EDITING);
				
				updateLists();
				
			} // end if to_edit != null
			
		}
		
	} // end editRemotePlatformProfile()
	
/***********************************************************************
 *
 * delListItem()
 *
 * @param list. The List to remove a list item from
 * @param jlist. The list that has the element selected
 *
 ***********************************************************************/
	private void delListItem(java.util.List list, JList jlist)
	{
		
		int index = jlist.getSelectedIndex();
		
		if (index != -1)
		{
			
			list.remove(index);
			
			this.updateList(list, jlist);
			
		}
		
	} // end delListItem()
	
/***********************************************************************
 *
 * addInternalTransport()
 *
 * Creates a new internal transport item and opens the editor for it
 *
 ***********************************************************************/
	private void addInternalTransport()
	{
		
		if (_pane == ACC_PANE)
		{
			InternalAddress addr = new InternalAddressImpl();
			
			_acc_internal_transport_addresses.add(addr);
			
			DataBindingEditor editor = new DataBindingEditor(this, addr, ADDING);
			
			updateLists();
			
		}
		else
		{
			InternalAddress addr = new InternalAddressImpl();
			
			_default_internal_transport_addresses.add(addr);
			
			DataBindingEditor editor = new DataBindingEditor(this, addr, ADDING);
			
			updateLists();
		}
		
	} // end addInternalTransport()
	
/***********************************************************************
 *
 * editInternalTransport()
 *
 * Creates an editor for the selected internal transport item
 *
 ***********************************************************************/
	private void editInternalTransport()
	{
		
		if (_pane == ACC_PANE)
		{
			
			int index = _jli_acc_internal_transports.getSelectedIndex();
			
			if (index != -1)
			{
				
				InternalAddress addr = (InternalAddressImpl)(_acc_internal_transport_addresses.get(index));
				
				if (addr != null)
				{
					
					DataBindingEditor editor = new DataBindingEditor(this, addr, EDITING);
					
					updateLists();
					
				} // end if transport != null
				
			}
			
		}
		else if (_pane == DEFAULT_PANE)
		{
			
			int index = _jli_default_internal_transports.getSelectedIndex();
			
			if (index != -1)
			{
				
				InternalAddress addr = (InternalAddressImpl)(_default_internal_transport_addresses.get(index));
				
				if (addr != null)
				{
					DataBindingEditor editor = new DataBindingEditor(this, addr, EDITING);
					
					updateLists();
					
				} // end if transport != null
				
			}
			
		} // end if _pane
		
	} // end editInternalTransport()
	
/***********************************************************************
 *
 * addExternalTransport()
 *
 * Creates a new external transport item and opens the editor for it
 *
 ***********************************************************************/
	private void addExternalTransport()
	{
		
		ExternalAddress addr = new ExternalAddressImpl();
		
		_acc_external_transport_addresses.add(addr);
		
		DataBindingEditor editor = new DataBindingEditor(this, addr, ADDING);
		
		updateLists();
		
	} // end addExternalTransport()
	
/***********************************************************************
 *
 * editExternalTransport()
 *
 * Creates an editor for the selected external transport item
 *
 ***********************************************************************/
	private void editExternalTransport()
	{
		
		int index = _jli_acc_external_transports.getSelectedIndex();
		
		if (index != -1)
		{
			
			ExternalAddress addr = (ExternalAddressImpl)(_acc_external_transport_addresses.get(index));
			
			if (addr != null)
			{
				
				DataBindingEditor editor = new DataBindingEditor(this, addr, ADDING);
				
				updateLists();
				
			} // end if transport != null
			
		}
		
	} // end editExternalTransport()
	
	
/***********************************************************************
 *
 * addProtocolMapping()
 *
 * Creates a new protocol mapping object when the add button is
 * pressed on the protocol mappings list.
 *
 ***********************************************************************/
	private void addProtocolMapping()
	{
		
		ProtocolMapping mapping = new ProtocolMappingImpl();
		
		_default_protocol_mappings.add(mapping);
		
		DataBindingEditor editor = new DataBindingEditor(this, mapping, ADDING);
		
		updateLists();
		
	} // end addProtocolMapping()
	
/***********************************************************************
 *
 * editProtocolMapping()
 *
 * Edits one of the protocol mappings when the edit button is
 * pressed on the protocol mappings list.
 *
 ***********************************************************************/
	private void editProtocolMapping()
	{
		
		int index = _jli_default_protocol_mappings.getSelectedIndex();
		
		if (index != -1)
		{
			
			ProtocolMapping to_edit = (ProtocolMapping)(_default_protocol_mappings.get(index));
			
			if (to_edit != null)
			{
				
				DataBindingEditor editor = new DataBindingEditor(this, to_edit, EDITING);
				
				updateLists();
				
			} // end if to_edit != null
			
		}
		
	} // end editProtocolMapping()
	
	
/***********************************************************************
 *
 * addLoaderAgentDescription()
 *
 * Creates a new agent description object when the add button is
 * pressed on the agent loader descriptions list.
 *
 ***********************************************************************/
	private void addLoaderAgentDescription()
	{
		
		AgentDescription desc = new AgentDescriptionImpl();
		
		_loader_agent_mappings.add(desc);
		
		DataBindingEditor editor = new DataBindingEditor(this, desc, ADDING);
		
		updateLists();
		
	} // end addLoaderAgentDescription()
	
/***********************************************************************
 *
 * editLoaderAgentDescription()
 *
 * Edits one of the agent description objects when the edit button is
 * pressed on the agent loader descriptions list.
 *
 ***********************************************************************/
	private void editLoaderAgentDescription()
	{
		
		int index = _jli_loader_agent_mappings.getSelectedIndex();
		
		if (index != -1)
		{
			
			AgentDescription to_edit = (AgentDescription)(_loader_agent_mappings.get(index));
			
			if (to_edit != null)
			{
				
				DataBindingEditor editor = new DataBindingEditor(this, to_edit, EDITING);
				
				updateLists();
				
			} // end if to_edit != null
			
		}
		
	} // end editLoaderAgentDescription()	
	
/***********************************************************************
 *
 * saveNewValues()
 *
 * Writes the values from the gui elements back to the main configurator
 * and gives the instruction to save the new profiles to disk.
 *
 ***********************************************************************/
	private void saveNewValues()
	{
		
		System.out.println("Saving");
		
		updatePlatformProfile();
		updateACCProfile();
		updateDefaultProfile();
		updateRuntimeProfile();
		updateLoaderProfile();
		
		// Save to disk
		
		try
		{
			_owner.saveProfile(_platform_profile);
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
		
		try
		{
			_owner.saveProfile(_acc_profile);
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
		
		try
		{
			_owner.saveProfile(_default_profile);
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
		
		try
		{
			_owner.saveProfile(_loader_profile);
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}		
		
		try
		{
			_owner.saveProfile(_runtime_profile);
		}
		catch(Throwable t)
		{
			t.printStackTrace();
		}
		
	} // end saveNewValues()
	
	private void updatePlatformProfile()
	{
		
		_platform_profile = new PlatformProfileImpl();
		
		_platform_profile.setAMSAddress(_jtf_ams_url.getText());
		_platform_profile.setHAPName(_jtf_hap_name.getText());
		_platform_profile.setProfileDirectory(_jtf_profile_directory.getText());
		
	} // end updatePlatformProfile()
	
	private void updateACCProfile()
	{
		
		_acc_profile = new ACCProfileImpl();
		
		_acc_profile.setLocalAddressesLocation(_jtf_local_address_location.getText());
		
		if (_acc_internal_transport_addresses != null);
		
		Iterator int_iter = _acc_internal_transport_addresses.iterator();
		
		while (int_iter.hasNext())
		{
			_acc_profile.addInternalAddress((InternalAddress)int_iter.next());
		}
		
		Iterator ext_iter = _acc_external_transport_addresses.iterator();
		
		while (ext_iter.hasNext())
		{
			_acc_profile.addExternalAddress((ExternalAddress)ext_iter.next());
		}
		
		Iterator remote_iter = _remote_platform_profiles.iterator();
		
		while (remote_iter.hasNext())
		{
			_acc_profile.addRemoteAgentPlatformProfile((RemoteAgentPlatformProfile)remote_iter.next());
		}
		
		DatabaseProfile acc_db = new DatabaseProfileImpl();
		
		acc_db.setDatabaseType((String)_jcb_acc_database_type.getSelectedItem());
		acc_db.setDatabaseLocation(_jtf_acc_database_location.getText());
		
		_acc_profile.setDatabaseProfile(acc_db);
		
	} // end updateACCProfile()
	
	private void updateDefaultProfile()
	{
		
		_default_profile = new AgentProfileImpl();
		
		Iterator int_iter = _default_internal_transport_addresses.iterator();
		
		while (int_iter.hasNext())
		{
			_default_profile.addInternalAddress((InternalAddress)int_iter.next());
		}
		
		Iterator mapping_iter = _default_protocol_mappings.iterator();
		
		ProtocolProfile protocol_profile = new ProtocolProfileImpl();
		
		while (mapping_iter.hasNext())
		{
			protocol_profile.addProtocolMapping((ProtocolMapping)mapping_iter.next());
		}
		
		_default_profile.setProtocolProfile(protocol_profile);
		
		DatabaseProfile default_db = new DatabaseProfileImpl();
		
		default_db.setDatabaseType((String)_jcb_default_database_type.getSelectedItem());
		default_db.setDatabaseLocation(_jtf_default_database_location.getText());
		
		_default_profile.setDatabaseProfile(default_db);
		
		_default_profile.setTaskManagerProfile(_tm_profile);
		
	} // end updateDefaultProfile()
	
	private void updateLoaderProfile()
	{
		
		_loader_profile = new LoaderProfileImpl();
		
		Iterator mapping_iter = _loader_agent_mappings.iterator();
		
		while (mapping_iter.hasNext())
		{
			_loader_profile.addAgentDescription((AgentDescription)mapping_iter.next());
		}
				
	} // end updateLoaderProfile()	
	
	private void updateRuntimeProfile() {
		
		_runtime_profile.setFipaosPath(_jtf_fipa_os_location.getText());
		_runtime_profile.setJdkPath(_jtf_jdk_location.getText());
		_runtime_profile.setFipaosVer(_jtf_fipa_os_ver.getText());
		_runtime_profile.setPlatformPath(_jtf_platform_profile_location.getText());
		
		_runtime_profile.setDebugger((String)(_jcb_debug_option.getSelectedItem()) == DEBUG_KEY1);
		if ((String)(_jcb_jit_option.getSelectedItem()) == JIT_KEY1)
			_runtime_profile.setNoJit(ScriptVars.JIT_NORMAL);
		else if((String)(_jcb_jit_option.getSelectedItem()) == JIT_KEY2)
			_runtime_profile.setNoJit(ScriptVars.JIT_NOJIT);
		else if((String)(_jcb_jit_option.getSelectedItem()) == JIT_KEY3)
			_runtime_profile.setNoJit(ScriptVars.JIT_NOHOTSPOT);
		_runtime_profile.setHasGui((String)(_jcb_debug_option.getSelectedItem()) == LOADER_GUI_KEY1);
		
		try {
			_runtime_profile.setDebugMemLvl(((Integer)_dbg_lvl_mappings.get((String)_jcb_dbg_lvl_mem.getSelectedItem())));
			_runtime_profile.setDebugFileLvl(((Integer)_dbg_lvl_mappings.get((String)_jcb_dbg_lvl_file.getSelectedItem())));
		}
		catch(Throwable t) {
			t.printStackTrace();
		}
		
	} // end updateRuntimeProfile()
	
/***********************************************************************
 *
 * updateLists()
 *
 * Updates the gui lists after a change
 *
 ***********************************************************************/
	private void updateLists()
	{
		
		this.updateList(_remote_platform_profiles, _jli_remote_profiles);
		this.updateList(_acc_internal_transport_addresses, _jli_acc_internal_transports);
		this.updateList(_acc_external_transport_addresses, _jli_acc_external_transports);
		this.updateList(_default_internal_transport_addresses, _jli_default_internal_transports);
		this.updateList(_default_protocol_mappings, _jli_default_protocol_mappings);
		this.updateList(_loader_agent_mappings, _jli_loader_agent_mappings);		
		
		this.pack();
		
	}
	
/***********************************************************************
 *
 * centre()
 *
 * centres the gui on screen
 *
 ***********************************************************************/
	private void confirmExit()
	{
		
		int save = JOptionPane.showConfirmDialog(null,
		"Save Settings Before Exiting?",
		"Save Settings?",
		JOptionPane.YES_NO_CANCEL_OPTION);
		
		if (save == JOptionPane.YES_OPTION)
		{
			
			this.saveNewValues();
			
		}
		else if (save == JOptionPane.CANCEL_OPTION)
		{
			
			return;
			
		}
		
		System.out.println("Exiting");
		
		this.dispose();
		System.exit(0);
		
	} // end confirmExit()
	
/***********************************************************************
 *
 * centre()
 *
 * centres the gui on screen
 *
 ***********************************************************************/
	private void centre()
	{
		
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		Dimension frameSize = this.getSize();
		
		this.setLocation((screenSize.width  - frameSize.width) / 2,
		(screenSize.height - frameSize.height) / 2);
		
	}
	
} // end FIPAOSConfiguratorGUI.java