//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s):
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved.
//
//******************************************************************************

package fipaos.tool.configurator;

import java.io.*;
import java.net.*;

/**********************************************************************
 *
 * FIPAOSConfiguratorTools
 *
 * Contains file handling tools used by the configurator
 *
 **********************************************************************/
public class FIPAOSConfiguratorTools {
	
	/**********************************************************************
	 *
	 * loadURLIntoString()
	 *
	 * @param String. The URL to load
	 *
	 * @return String. The contents of the URL
	 *
	 **********************************************************************/
	public static String loadURLIntoString(String url) throws FileNotFoundException, IOException
	{
		
		// Attempt to open a connection to the URL
		URLConnection connection = new java.net.URL( url ).openConnection();
		BufferedReader reader = new BufferedReader(
		new InputStreamReader(
		connection.getInputStream()
		)
		);
		
		return readIntoString( reader );
	} // end loadFileIntoString()
	
	/**********************************************************************
	 *
	 * loadFileIntoString()
	 *
	 * @param String. The path of the file to load
	 *
	 * @return String. The contents of the file
	 *
	 **********************************************************************/
	public static String loadFileIntoString(String filename) throws FileNotFoundException, IOException
	{
		
		BufferedReader reader = new BufferedReader(new FileReader(filename));
		
		return readIntoString( reader );
	} // end loadFileIntoString()
	
	/**********************************************************************
	 *
	 * readIntoString()
	 *
	 * @param BufferedReader. The buffered reader from which to read
	 *
	 * @return String. The contents read from the reader
	 *
	 **********************************************************************/
	private static String readIntoString(BufferedReader reader) throws FileNotFoundException, IOException
	{
		
		StringBuffer buffer = new StringBuffer();
		
		String line = null;
		
		line = reader.readLine();
		
		boolean first_time = true;
		
		while (line != null) {
			
			if( !first_time )
			{
				buffer.append('\n');
			}
			else
			{
				first_time = false;
			}
			
			buffer.append(line);
			
			line = reader.readLine();
			
		} // end while
		
		reader.close();
		
		return buffer.toString();
		
	} // end loadFileIntoString()
	
	/**********************************************************************
	 *
	 * saveString()
	 *
	 * @param BufferedWriter. The writer into which the contents are to be saved
	 *
	 * @param String. The contents to save
	 *
	 **********************************************************************/
	private static void saveString(BufferedWriter writer, String contents) throws java.io.IOException
	{
		writer.write(contents, 0, contents.length());
		writer.close();
		
	} // end saveStringIntoFile()
	
	/**********************************************************************
	 *
	 * saveStringIntoURL()
	 *
	 * @param String. The URL to save into
	 *
	 * @param String. The contents to save
	 *
	 **********************************************************************/
	public static void saveStringIntoURL(String url, String contents) throws java.io.IOException
	{
		
		// Attempt to open a connection to the URL
		URLConnection connection = new java.net.URL( url ).openConnection();
		BufferedWriter writer = new BufferedWriter(
		new OutputStreamWriter(connection.getOutputStream()));
		
		saveString( writer, contents );
		
	} // end saveStringIntoURL()
	
	/**********************************************************************
	 *
	 * saveStringIntoFile()
	 *
	 * @param String. The filename to save as
	 *
	 * @param String. The contents to save
	 *
	 **********************************************************************/
	public static void saveStringIntoFile(String filename, String contents) throws java.io.IOException
	{
		
		BufferedWriter writer = null;
		
		writer = new BufferedWriter(new FileWriter(filename));
		
		saveString( writer, contents );
		
	} // end saveStringIntoFile()
	
	/**********************************************************************
	 *
	 * saveStringIntoFileAndPath()
	 *
	 * @param String. The filename to save as. The path is created if it does not exist
	 *
	 * @param String. The contents to save
	 *
	 **********************************************************************/
	public static void saveStringIntoFileAndPath(String path, String filename, String contents) throws java.io.IOException
	{
		
		BufferedWriter writer = null;
		
		File fullpath = new File(path);
		
		fullpath.mkdirs();
		
		writer = new BufferedWriter(new FileWriter(new File(fullpath, filename)));
		
		saveString( writer, contents );
		
	} // end saveStringIntoFileAndPath()
	
	/***************************************************************************
	 *
	 * getFilenames()
	 *
	 * @param directory. A String containing the directory to search in.
	 *
	 * @return String[] containing the filenames found in the given directory.
	 *
	 ***************************************************************************/
	public static String[] getFilenames(String directory)
	{
		
		File dummy = new File(directory);
		
		return dummy.list();
		
	} // end getFilenames()
	
	/**********************************************************************
	 *
	 * main()
	 *
	 **********************************************************************/
	public static void main(String args[]) throws Exception {
		
		String test = FIPAOSConfiguratorTools.loadFileIntoString(args[0]);
		
		FIPAOSConfiguratorTools.saveStringIntoFile(args[1], test);
		
	} // end main()
	
} // end FIPAOSConfiguratorTools.java
