//******************************************************************************
//
//  Open Source Copyright Notice and License: FIPA-OS
//
//  The contents of this file are subject to the Emorphia Public License (the
//  "License"); you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at http://www.emorphia.com/EPL/
//
//  Software distributed under the License is distributed on an "AS IS" basis,
//  WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
//  for the specific language governing rights and limitations under the
//  License.
//
//  The Original Code is FIPA-OS.
//
//  The Initial Developer of the Original Code is Emorphia.
//
//  Portions created by Emorphia are Copyright (C) 2001 Emorphia Limited
//  licensed under the Emorphia Public License, available at
//  http://www.emorphia.com/EPL/.
//
//  All Rights Reserved.
//
//  Contributor(s):
//  (add contributor names here)
//
//******************************************************************************

package fipaos.tool.wizard;

import fipaos.tool.wizard.panels.*;
import fipaos.agent.profile.*;
import fipaos.agent.dataMapper.*;
import fipaos.tool.configurator.*;
import fipaos.util.DIAGNOSTICS;

import javax.swing.*;
import java.io.*;
import java.util.*;

public class FIPAOSWizard extends javax.swing.JFrame implements ScriptVars
{
	// Array of TransportDefinition that will be used by the various Wizard
	// GUI panels
	public final static TransportDefinition[] TRANSPORTS;
	
	public final static String STANDALONE_AP_NAME 	= "localap";
	
	private static final String PROFILES 		= "profiles";
	private static final String PLATFORM_PROFILE 	= "platform.profile";
	private static final String ACC_PROFILE 	= "acc.profile";
	private static final String DEFAULT_PROFILE 	= "default.profile";
	private static final String LOADER_PROFILE 	= "loader.profile";
	private static final String FIPAOS_LOCATION 	= System.getProperty( "fipaos.home", ".." );
	
	static
	{
		
		TransportDefinition td_fipaos_rmi = new TransportDefinition();
		td_fipaos_rmi.setName("fipaos-rmi");
		td_fipaos_rmi.setExternal(false);
		td_fipaos_rmi.setUsedByAgent(false);
		td_fipaos_rmi.setUsedByACC(false);		
		td_fipaos_rmi.setNSPort(3000);
		td_fipaos_rmi.setScriptNSEnableVar(STARTFIPAOS_STARTRMINS);
		td_fipaos_rmi.setScriptNSPortVar(STARTFIPAOS_RMINSPORT);
		
		
		TransportDefinition td_fipaos_ssl_rmi = new TransportDefinition();
		td_fipaos_ssl_rmi.setName("fipaos-ssl-rmi");
		td_fipaos_ssl_rmi.setExternal(false);
		td_fipaos_ssl_rmi.setUsedByAgent(false);
		td_fipaos_ssl_rmi.setUsedByACC(false);
		td_fipaos_ssl_rmi.setNSPort(3000);
		td_fipaos_ssl_rmi.setScriptNSEnableVar(STARTFIPAOS_STARTSSLRMINS);
		td_fipaos_ssl_rmi.setScriptNSPortVar(STARTFIPAOS_SSLRMINSPORT);
		
		TransportDefinition td_fipaos_corbaname = new TransportDefinition();
		td_fipaos_corbaname.setName("corbaname");
		td_fipaos_corbaname.setExternal(true);
		td_fipaos_corbaname.setUsedByAgent(false);
		td_fipaos_corbaname.setUsedByACC(false);
		td_fipaos_corbaname.setNSPort(4000);
		td_fipaos_corbaname.setScriptNSEnableVar(STARTFIPAOS_STARTIIOPNS);
		td_fipaos_corbaname.setScriptNSPortVar(STARTFIPAOS_IIOPNSPORT);
		
		TransportDefinition td_fipaos_http = new TransportDefinition();
		td_fipaos_http.setName("http");
		td_fipaos_http.setExternal(true);
		td_fipaos_http.setUsedByAgent(false);
		td_fipaos_http.setUsedByACC(false);
		td_fipaos_http.setNSPort(8080);
		td_fipaos_http.setScriptNSEnableVar(null);
		td_fipaos_http.setScriptNSPortVar(null);
		
		TRANSPORTS = new TransportDefinition[]
		{td_fipaos_rmi, td_fipaos_ssl_rmi, td_fipaos_corbaname, td_fipaos_http};
		
	}
	
	private PlatformProfile _platform_profile;
	private AgentProfile _default_profile;
	private ACCProfile _acc_profile;
	private LoaderProfile _loader_profile;
	private ScriptEditor _startfipaos_script;
	private ScriptEditor _setupfipaos_script;
	private Map _startfipaos_vars;
	private Map _setupfipaos_vars;
	
	private List _panels = new LinkedList( Arrays.asList( new WizardPanel[]
	{ new IntroductionPanel(), new PlatformDistribution( this ), new EndPanel() } ) );
	
	private int _current_pane = 0;
	
	public FIPAOSWizard()
	{
		
		initComponents();
		initialiseWizardPanels();
		setCurrentPane( (JComponent) _panels.get( 0 ) );
		pack();
		
		// Resize and reposition window so it appears in the center
		// of the screen, and is not too large
		java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
		int width = getSize().width;
		int height = getSize().height;
		width = 640;
		height = 500;
		if( tk.getScreenSize().width < width )
		{
			width = tk.getScreenSize().width;
		}
		if( tk.getScreenSize().height < height )
		{
			height = tk.getScreenSize().height;
		}
		
		setSize( width, height );
		width = tk.getScreenSize().width - width;
		height = tk.getScreenSize().height - height;
		setLocation( width/2, height/2 );
		
		setVisible( true );
		makeButtonsAvailable();
	}
	
		/**
		 * This method is called from within the constructor to initialize the form.
		 * WARNING: Do NOT modify this code. The content of this method is always regenerated by the FormEditor.
		 */
	private void initComponents()//GEN-BEGIN:initComponents
	{
		jPanel1 = new javax.swing.JPanel();
		_cancel = new javax.swing.JButton();
		_prev = new javax.swing.JButton();
		_next = new javax.swing.JButton();
		_finish = new javax.swing.JButton();
		jSeparator1 = new javax.swing.JSeparator();
		jPanel2 = new javax.swing.JPanel();
		jLabel1 = new javax.swing.JLabel();
		jSeparator2 = new javax.swing.JSeparator();
		jPanel3 = new javax.swing.JPanel();
		jLabel2 = new javax.swing.JLabel();
		jScrollPane2 = new javax.swing.JScrollPane();
		jPanel4 = new javax.swing.JPanel();
		
		setIconImage(new ImageIcon( ClassLoader.getSystemResource( "images/logo.gif" ) ).getImage());
		setResizable(false);
		addWindowListener(new java.awt.event.WindowAdapter()
		{
			public void windowClosing(java.awt.event.WindowEvent evt)
			{
				exitForm(evt);
			}
		});
		
		jPanel1.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints1;
		
		_cancel.setText("Cancel");
		_cancel.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				cancel(evt);
			}
		});
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 1;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.insets = new java.awt.Insets(5, 5, 5, 10);
		gridBagConstraints1.anchor = java.awt.GridBagConstraints.EAST;
		gridBagConstraints1.weightx = 1.0;
		jPanel1.add(_cancel, gridBagConstraints1);
		
		_prev.setText("< Prev");
		_prev.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				prev(evt);
			}
		});
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 2;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.insets = new java.awt.Insets(5, 1, 5, 1);
		jPanel1.add(_prev, gridBagConstraints1);
		
		_next.setText("Next >");
		_next.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				next(evt);
			}
		});
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 3;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.insets = new java.awt.Insets(5, 1, 5, 1);
		jPanel1.add(_next, gridBagConstraints1);
		
		_finish.setText("Finish");
		_finish.addActionListener(new java.awt.event.ActionListener()
		{
			public void actionPerformed(java.awt.event.ActionEvent evt)
			{
				finish(evt);
			}
		});
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 4;
		gridBagConstraints1.gridy = 1;
		gridBagConstraints1.insets = new java.awt.Insets(5, 10, 5, 5);
		jPanel1.add(_finish, gridBagConstraints1);
		
		gridBagConstraints1 = new java.awt.GridBagConstraints();
		gridBagConstraints1.gridx = 0;
		gridBagConstraints1.gridy = 0;
		gridBagConstraints1.gridwidth = java.awt.GridBagConstraints.REMAINDER;
		gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints1.insets = new java.awt.Insets(5, 5, 5, 5);
		gridBagConstraints1.weightx = 1.0;
		jPanel1.add(jSeparator1, gridBagConstraints1);
		
		getContentPane().add(jPanel1, java.awt.BorderLayout.SOUTH);
		
		jPanel2.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints2;
		
		jLabel1.setText("Configuration Wizard");
		jLabel1.setHorizontalAlignment(javax.swing.SwingConstants.RIGHT);
		jLabel1.setFont(new java.awt.Font("Dialog", 0, 24));
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridx = 0;
		gridBagConstraints2.gridy = 0;
		gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
		gridBagConstraints2.insets = new java.awt.Insets(5, 5, 5, 5);
		gridBagConstraints2.anchor = java.awt.GridBagConstraints.EAST;
		gridBagConstraints2.weightx = 1.0;
		jPanel2.add(jLabel1, gridBagConstraints2);
		
		gridBagConstraints2 = new java.awt.GridBagConstraints();
		gridBagConstraints2.gridwidth = java.awt.GridBagConstraints.REMAINDER;
		gridBagConstraints2.fill = java.awt.GridBagConstraints.HORIZONTAL;
		gridBagConstraints2.insets = new java.awt.Insets(5, 5, 5, 5);
		jPanel2.add(jSeparator2, gridBagConstraints2);
		
		getContentPane().add(jPanel2, java.awt.BorderLayout.NORTH);
		
		jPanel3.setLayout(new java.awt.GridBagLayout());
		java.awt.GridBagConstraints gridBagConstraints3;
		
		jLabel2.setIcon(new ImageIcon( ClassLoader.getSystemResource( "images/wizard-banner.jpg" ) ));
		gridBagConstraints3 = new java.awt.GridBagConstraints();
		gridBagConstraints3.insets = new java.awt.Insets(0, 5, 0, 5);
		gridBagConstraints3.anchor = java.awt.GridBagConstraints.NORTH;
		jPanel3.add(jLabel2, gridBagConstraints3);
		
		jScrollPane2.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(1, 1, 1, 1)));
		jScrollPane2.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		jScrollPane2.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		gridBagConstraints3 = new java.awt.GridBagConstraints();
		gridBagConstraints3.gridwidth = java.awt.GridBagConstraints.REMAINDER;
		gridBagConstraints3.gridheight = java.awt.GridBagConstraints.REMAINDER;
		gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints3.insets = new java.awt.Insets(5, 5, 5, 5);
		gridBagConstraints3.weightx = 1.0;
		gridBagConstraints3.weighty = 1.0;
		jPanel3.add(jScrollPane2, gridBagConstraints3);
		
		jPanel4.setBackground(new java.awt.Color(51, 51, 255));
		gridBagConstraints3 = new java.awt.GridBagConstraints();
		gridBagConstraints3.gridx = 0;
		gridBagConstraints3.gridy = 1;
		gridBagConstraints3.fill = java.awt.GridBagConstraints.BOTH;
		gridBagConstraints3.insets = new java.awt.Insets(0, 5, 0, 5);
		gridBagConstraints3.anchor = java.awt.GridBagConstraints.NORTH;
		gridBagConstraints3.weighty = 1.0;
		jPanel3.add(jPanel4, gridBagConstraints3);
		
		getContentPane().add(jPanel3, java.awt.BorderLayout.CENTER);
		
	}//GEN-END:initComponents
	
  private void cancel(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancel
	  dispose();
	  System.exit( 0 );
  }//GEN-LAST:event_cancel
  
  private void prev(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_prev

		// Execute prev() in new Thread
		Thread t = new Thread( new Runnable()
		{
			public void run()
			{
				prev();
			}
		} );
		t.start();
			
  }//GEN-LAST:event_prev
  
  private void next(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_next

		// Execute next() in new Thread
		Thread t = new Thread( new Runnable()
		{
			public void run()
			{
			  next();
			}
		} );
		t.start();
		
  }//GEN-LAST:event_next
  
  private void finish(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_finish
	  // Check if it is OK to continue
	  
 		// Execute in new Thread
		Thread t = new Thread( new Runnable()
		{
			public void run()
			{
				WizardPanel wp = (WizardPanel) _panels.get( _current_pane );
				if( !wp.isOk() )
				{
					return;
				}

				saveIfModified();
				dispose();
				System.exit( 0 );
			}
		} );
		t.start();
		
  }//GEN-LAST:event_finish
  
		/** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) { //GEN-FIRST:event_exitForm
	    cancel( null );
    } //GEN-LAST:event_exitForm
    
		/** @param args the command line arguments */
    public static void main(String[] args)
    {
	    // Attempt to set the default L&F for the platform we're running on
	    try
	    {
		    UIManager.setLookAndFeel ( UIManager.getSystemLookAndFeelClassName() );
	    }
	    catch( Throwable t )
	    {
		    DIAGNOSTICS.println( "Couldn't set the default Look & Feel for this OS", DIAGNOSTICS.LEVEL_MAX);
	    }
	    
	    // Set the default JTextArea background color to match window color
	    try
	    {
		    LookAndFeel laf = UIManager.getLookAndFeel();
		    Object bg_color = UIManager.get( "Label.background" );
		    UIManager.put( "TextArea.background", bg_color );
		    UIManager.put( "Panel.background", bg_color );
	    }
	    catch( Throwable t )
	    {
		    DIAGNOSTICS.println( "Couldn't set default background color", DIAGNOSTICS.LEVEL_MAX);
	    }
	    
	    new FIPAOSWizard().show();
    }
    
    private void setCurrentPane(JComponent pane)
    {
	    jScrollPane2.setViewportView( pane );
    }
    
    private void prev()
    {
	    if( _current_pane > 0 )
	    {
		    _current_pane--;
	    }
	    
	    makeButtonsAvailable();
	    
	    setCurrentPane( (JComponent) _panels.get( _current_pane ) );
    }
    
    private void next()
    {
	    // Check OK to continue
	    WizardPanel wp = (WizardPanel) _panels.get( _current_pane );
	    if( !wp.isOk() )
	    {
		    return;
	    }
	    
	    if( _current_pane < ( _panels.size() - 1 ) )
	    {
		    _current_pane++;
	    }
	    
	    makeButtonsAvailable();
	    
	    setCurrentPane( (JComponent) _panels.get( _current_pane ) );
    }
    
    private void makeButtonsAvailable()
    {
	    boolean at_end = _current_pane >= ( _panels.size() -1 );
	    _prev.setEnabled( _current_pane > 0 );
	    _next.setEnabled( !at_end );
	    _finish.setEnabled( at_end );
    }
    
		/**
		 * Inserts the a WizardPanel after another WizardPanel in the list of panels
		 * to display
		 */
    public void insertPanel( WizardPanel insert_after, WizardPanel insert )
    {
	    DIAGNOSTICS.println( "insertPanel()" );
	    // Get index of panel to insert after
	    int index = _panels.indexOf( insert_after );
	    index++;
	    
	    // Increment _current_pane if necessary (i.e. if current pane will change index)
	    if( index <= _current_pane )
	    {
		    _current_pane++;
	    }
	    
	    _panels.add( index, insert );
	    insert.initialise( _acc_profile, _default_profile, _loader_profile, _platform_profile,
	    _setupfipaos_vars, _startfipaos_vars );
	    
	    makeButtonsAvailable();
    }
    
		/**
		 * Removes the given WizardPanel from the list of panels to display
		 */
    public void removePanel( WizardPanel remove )
    {
	    DIAGNOSTICS.println( "removePanel()" );
	    // Get index of panel to remove
	    int index = _panels.indexOf( remove );
	    
	    // Decrement _current_pane if necessary (i.e. if current pane will change index)
	    if( index <= _current_pane )
	    {
		    _current_pane--;
	    }
	    
	    _panels.remove( remove );
	    
	    makeButtonsAvailable();
    }
    
    
		/**
		 * Method to initialise all WizardPanel's
		 */
    private void initialiseWizardPanels()
    {
	    DIAGNOSTICS.println( "initialiseWizardPanels()" );
	    loadProfiles();
	    
	    // Iterate through WizardPanels', and initialise
	    Iterator i =(  new LinkedList( _panels ) ).iterator();
	    while( i.hasNext() )
	    {
		    WizardPanel wp = (WizardPanel) i.next();
		    wp.initialise( _acc_profile, _default_profile, _loader_profile, _platform_profile,
		    _setupfipaos_vars, _startfipaos_vars );
	    }
    }
    
		/**
		 * Check if any WizardPanel's have changed
		 */
    private boolean isModified()
    {
	    DIAGNOSTICS.println( "isModified()" );
	    
	    boolean changed = false;
	    
	    // Iterate through WizardPanels', and OR wp.isModified()
	    Iterator i = _panels.iterator();
	    while( i.hasNext() && !changed )
	    {
		    WizardPanel wp = (WizardPanel) i.next();
		    changed |= wp.isModified();
	    }
	    
	    return changed;
    }
    
		/**
		 * Reset changed flag on all WizardPanels
		 */
    private void resetModified()
    {
	    DIAGNOSTICS.println( "resetModified()" );
	    
	    // Iterate through WizardPanels', and OR wp.isModified()
	    Iterator i = _panels.iterator();
	    while( i.hasNext() )
	    {
		    WizardPanel wp = (WizardPanel) i.next();
		    wp.resetModified();
	    }
    }
    
		/**
		 * Check if anything has changed - if so save profiles
		 */
    private void saveIfModified()
    {
	    DIAGNOSTICS.println( "Check if we should save profiles etc.", DIAGNOSTICS.LEVEL_MAX );
	    
	    // Check if modified
	    if( isModified() )
	    {
		    DIAGNOSTICS.println( "Saving profiles etc.",DIAGNOSTICS.LEVEL_MAX );
		    saveProfiles();
		    resetModified();
	    }
	    else
	    {
		    DIAGNOSTICS.println( "Nothing modified, not saving",DIAGNOSTICS.LEVEL_MAX );
	    }
    }
    
		/**
		 * Fetch and initialise profiles
		 */
    private void loadProfiles()
    {
	    boolean first_run = false;
	    
	    // Fetch platform profile
	    File fipaos_dir = new File( FIPAOS_LOCATION );
	    File profiles_dir = new File( fipaos_dir, PROFILES );
	    File platform_file = new File( profiles_dir, PLATFORM_PROFILE );
	    _platform_profile = (PlatformProfile)fetchProfile( platform_file );
	    
	    // Check if platform profile has a profile-location specified
	    if( _platform_profile.getProfileDirectory() == null ||
	    _platform_profile.getProfileDirectory().equals( "" ) )
	    {
		    try
		    {
			    // Set profile location to be the same directory as the platform profile itself
			    _platform_profile.setProfileDirectory( profiles_dir.getCanonicalPath() + File.separator );
		    }
		    catch( Throwable t )
		    {
		    }
		    
		    first_run = true;
	    }
	    else
	    {
		    // Update profiles location appropriately
		    profiles_dir = new File( _platform_profile.getProfileDirectory() );
	    }
	    
	    // Load the other profiles
	    _acc_profile = (ACCProfile)fetchProfile( new File( profiles_dir, ACC_PROFILE ) );
	    _default_profile = (AgentProfile)fetchProfile( new File( profiles_dir, DEFAULT_PROFILE ) );
	    _loader_profile = (LoaderProfile)fetchProfile( new File( profiles_dir, LOADER_PROFILE ) );
	    
	    // Load batch files
	    File bat_dir = new File( fipaos_dir, "bat" );
	    
	    // Load SetupFIPAOS
	    try
	    {
		    File setupfipaos = new File( bat_dir, "SetupFIPAOS.bat" );
		    if( !setupfipaos.exists() )
		    {
			    setupfipaos = new File( bat_dir, "SetupFIPAOS" );
		    }
		    
		    if( !setupfipaos.exists() )
		    {
			    DIAGNOSTICS.println( "WARNING: Can't locate FIPA-OS batch file SetupFIPAOS" );
		    }
		    else
		    {
			    _setupfipaos_script = new ScriptEditor( setupfipaos, SETUPFIPAOS_VARS );
			    _setupfipaos_vars = _setupfipaos_script.getScriptVariables();
		    }
		    
		    // Load Go
		    File startfipaos = new File( bat_dir, "StartFIPAOS.bat" );
		    if( !startfipaos.exists() )
		    {
			    startfipaos = new File( bat_dir, "StartFIPAOS" );
		    }
		    
		    if( !startfipaos.exists() )
		    {
			    DIAGNOSTICS.println( "WARNING: Can't locate FIPA-OS batch file StartFIPAOS" );
		    }
		    else
		    {
			    _startfipaos_script = new ScriptEditor( startfipaos, STARTFIPAOS_VARS );
			    _startfipaos_vars = _startfipaos_script.getScriptVariables();
		    }
	    }
	    catch( IOException e )
	    {
		    DIAGNOSTICS.println( "Error when reading scripts, ignoring...." );
	    }
	    
	    // Initialise other stuff if this is the first run
	    if (first_run == true)
	    {
		    try
		    {
			    if (_acc_profile != null)
			    {
				    _acc_profile.setLocalAddressesLocation( (new File( fipaos_dir, "platform.addresses" )).getCanonicalPath() );
				    _acc_profile.getDatabaseProfile().setDatabaseLocation( (new File( fipaos_dir, "databases" )).getCanonicalPath() + File.separator );
			    }
			    if (_default_profile != null)
			    {
				    _default_profile.getDatabaseProfile().setDatabaseLocation( (new File( fipaos_dir, "databases" )).getCanonicalPath() + File.separator );
			    }
			    if ( _setupfipaos_vars != null )
			    {
				    _setupfipaos_vars.put( SETUPFIPAOS_FIPAOS, fipaos_dir.getCanonicalPath() );
			    }
			    
			    // Save all profiles!
			    saveProfiles();
		    }
		    catch( IOException e )
		    {
			    DIAGNOSTICS.println( "Error when updating scripts for first time, ignoring....", DIAGNOSTICS.LEVEL_MAX );
		    }
	    }
    }
    
		/**
		 * Save all profiles
		 */
    private void saveProfiles()
    {
	    File fipaos_dir = new File( FIPAOS_LOCATION );
	    File profiles_dir = new File( fipaos_dir, PROFILES );
	    
	    if( _acc_profile != null )
	    {
		    saveProfile( _acc_profile, new File( profiles_dir, ACC_PROFILE ) );
		    DIAGNOSTICS.println("Saving ACC profile", this, DIAGNOSTICS.LEVEL_2);
	    }
	    
	    if( _default_profile != null )
	    {
		    saveProfile( _default_profile, new File( profiles_dir, DEFAULT_PROFILE ) );
		    DIAGNOSTICS.println("Saving default profile", this, DIAGNOSTICS.LEVEL_2);
	    }
	    
	    if( _loader_profile != null )
	    {
		    saveProfile( _loader_profile, new File( profiles_dir, LOADER_PROFILE ) );
		    DIAGNOSTICS.println("Saving loader profile", this, DIAGNOSTICS.LEVEL_2);
	    }
	    
	    if( _platform_profile != null )
	    {
		    saveProfile( _platform_profile, new File( profiles_dir, PLATFORM_PROFILE ) );
		    DIAGNOSTICS.println("Saving platform profile", this, DIAGNOSTICS.LEVEL_2);
	    }
	    
	    if( _setupfipaos_script != null )
	    {
		    _setupfipaos_script.setScriptVariables( _setupfipaos_vars );
		    _setupfipaos_script.saveNewConfiguration();
		    DIAGNOSTICS.println("Saving setupFIPAOS script", this, DIAGNOSTICS.LEVEL_2);
	    }
	    
	    if( _startfipaos_script != null )
	    {
		    _startfipaos_script.setScriptVariables( _startfipaos_vars );
		    _startfipaos_script.saveNewConfiguration();
		    DIAGNOSTICS.println("Saving startFIPAOS script", this, DIAGNOSTICS.LEVEL_2);
	    }
    }
    
		/**
		 * Save the given profile
		 */
    private void saveProfile(Profile profile, File location )
    {
	    try
	    {
		DataMapper dataMapper = DataMapperFactory.createDataMapperByFile( location.getCanonicalPath(), null );
		if( dataMapper != null )
			dataMapper.marshall( profile );
	    }
	    catch( Throwable t )
	    {
		    t.printStackTrace();
	    }
    }
    
		/**
		 * Load a profile from the given location
		 */
    private Profile fetchProfile( File location )
    {
	    Profile profile = null;
	    
	    try
	    {
		    DataMapper dataMapper = DataMapperFactory.createDataMapperByFile( location.getCanonicalPath(), null );
		    profile = (Profile) dataMapper.unmarshall();
	    }
	    catch (Throwable t)
	    {
		    t.printStackTrace();
	    }
	    
	    return profile;
    }
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel jPanel1;
    private javax.swing.JButton _cancel;
    private javax.swing.JButton _prev;
    private javax.swing.JButton _next;
    private javax.swing.JButton _finish;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JSeparator jSeparator2;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JPanel jPanel4;
    // End of variables declaration//GEN-END:variables
}
