//******************************************************************************
//  Open Source Copyright Notice and License: FIPA-OS
//
//  1. The programs and other works made available to you in these files 
//     ("the Programs") are Copyright (c) 1999 - 2000 Nortel Networks
//     Corporation, 8200 Dixie Road, Suite 100, Brampton, Ontario, Canada 
//     L6R 5P6.  All rights reserved.
//
//  2. Your rights to copy, distribute and modify the Programs are as set out 
//     in the Nortel Networks FIPA-OS Public License, a copy of which can be
//     found in file "Nortel_FIPA_OS_Public_Licence.txt" and the latest version 
//     can also be found at http://fipa-os.sourceforge.net/. By  downloading
//     the files containing the Programs you accept the terms and conditions of
//     the Public License. You do not have to accept these terms and conditions,
//     but unless you do so you have no rights to use the Programs.
//
//  The Original Code is Nortel Networks' FIPA-OS (Foundation for Intelligent
//  Physical Agents - Open Source).
//
//  The Initial Developer of the Original Code is Nortel Networks Corporation.
//  Portions created by Nortel Networks Corporation or its subsidiaries are
//  Copyright (c) 1999 - 2000 Nortel Networks Corporation. All Rights Reserved.
//
//  Contributor(s): 
//    Emorphia agree to provide Modifications to Nortel Networks' FIPA-OS 
//    Covered Code under Nortel Networks' FIPA-OS Public License. All Emorphia's 
//    Modifications remain Copyright (c) 2001 Emorphia Limited. All Rights
//    Reserved. 
//
//******************************************************************************

package fipaos.util;

//----------------------------------------------------------------------

import java.net.*;
import java.io.*;

public class DocumentRetriever
{
	/**
	 * This varient of the getDoc method will attempt to load the given file from
	 * a class-path entry
	 */
	public static String getDocFromClasspath(String name)
	{
		StringBuffer document = new StringBuffer();
		
		BufferedReader reader;
		
		// Attempt to find the file and setup a DataInputStream
		try
		{
			InputStreamReader is_reader = new InputStreamReader( ClassLoader.getSystemResourceAsStream( name ) );
			reader = new BufferedReader( is_reader );
		}
		catch( Throwable e)
		{
			return null;
		}
		
		try
		{
			// Read in the document until its done!
			String line = reader.readLine();
			String newline = System.getProperty("line.separator");
			
			while(line != null)
			{
				document.append(line);
				document.append(newline);
				line = reader.readLine();
			}
		}
		catch( IOException e )
		{
			DIAGNOSTICS.println( "Exception occured whilst retreiving document: " + name + ", returning null" );
			return null;
		}
		
		return document.toString();		
	}
	
	/**
	 * This varient of the getDoc method will attempt to load the given file from
	 * a class-path entry
	 */
	public static byte[] getBinDocFromClasspath(String name)
	{
		byte[] document = null;
		
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		
		// Attempt to read the file
		try
		{
			InputStream is = ClassLoader.getSystemResourceAsStream( name );
			
			byte buff[] = new byte[1024];
			int next = is.read( buff );
			while( next > 0 )
			{
				os.write( buff, 0, next );
				next = is.read( buff );
			}
			
			is.close();
			os.close();
			document = os.toByteArray();
		}
		catch( Throwable e)
		{
		}
		
		return document;
	}
	
	/**
	 * This public getDoc method takes in the name of the documemt that is
	 * to be retrieved and will either return the document if successful or
	 * will return null after an exception is thrown.
	 */
	public static String getDocFromURL(String name)
	{
		StringBuffer document = new StringBuffer();
		
		try
		{
			// Attempt to create a URL from the given "name"
			java.net.URL url = new java.net.URL( name );
			
			// Attempt to open a connection to the URL
			URLConnection connection = url.openConnection();
			BufferedReader in = new BufferedReader(
			new InputStreamReader(
			connection.getInputStream()
			)
			);
			
			
			// Read in the contents of the connection/URL
			String next_line = in.readLine();
			String newline = System.getProperty("line.separator");
			
			while(next_line != null)
			{
				document.append(next_line);
				document.append(newline);
				next_line = in.readLine();
			}
		}
		catch(Throwable f)
		{
			return null;
		}
		
		return document.toString();
	}
	
	/**
	 * This public getDoc method takes in the name of the documemt that is
	 * to be retrieved and will either return the document if successful or
	 * will return null after an exception is thrown.
	 */
	public static byte[] getBinDocFromURL(String name)
	{
		byte[] document = null;
		
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		
		// Attempt to read the file
		try
		{
			// Attempt to create a URL from the given "name"
			java.net.URL url = new java.net.URL( name );
			
			// Attempt to open a connection to the URL
			URLConnection connection = url.openConnection();
			InputStream is = connection.getInputStream();
			
			byte buff[] = new byte[1024];
			int next = is.read( buff );
			while( next > 0 )
			{
				os.write( buff, 0, next );
				next = is.read( buff );
			}
			
			is.close();
			os.close();
			document = os.toByteArray();
		}
		catch( Throwable e)
		{
		}
		
		return document;
	}
	
	/**
	 * This public getDoc method takes in the name of the documemt that is
	 * to be retrieved and will either return the document if successful or
	 * will return null after an exception is thrown.
	 */
	public static String getDocFromFile(String name)
	{
		StringBuffer document = new StringBuffer();
		
		// Attempt to find the file and setup a DataInputStream
		try
		{
			BufferedReader reader = new BufferedReader(new FileReader(name));
			
			// Read in the document until its done!
			String line = reader.readLine();
			String newline = System.getProperty("line.separator");
			
			while(line != null)
			{
				document.append(line);
				document.append(newline);
				line = reader.readLine();
			}
		}
		catch( Throwable e)
		{
			return null;
		}
		
		return document.toString();
	}
	
	/**
	 * This public getDoc method takes in the name of the documemt that is
	 * to be retrieved and will either return the document if successful or
	 * will return null after an exception is thrown.
	 */
	public static byte[] getBinDocFromFile(String name)
	{
		byte[] document = null;
		
		ByteArrayOutputStream os = new ByteArrayOutputStream();
				
		// Attempt to read the file
		try
		{
			InputStream is =  new FileInputStream(name);
			
			byte buff[] = new byte[1024];
			int next = is.read( buff );
			while( next > 0 )
			{
				os.write( buff, 0, next );
				next = is.read( buff );
			}
			
			is.close();
			os.close();
			document = os.toByteArray();
		}
		catch( Throwable e)
		{
			return null;
		}
		
		return document;
	}
	
	/**
	 * This method takes in the name of the documemt that is
	 * to be retrieved and will either return the document if successful or
	 * will return null after an exception is thrown.  This method attempts to use
	 * the other getDocFromX() methods of this class to retrieve the given document
	 */
	public static String getDoc(String name)
	{
		// try to fetch from URL first:
		
		String doc = getDocFromURL(name);
		
		if (doc == null)
		{
			doc = getDocFromFile(name);
		}
		
		if (doc == null)
		{
			doc = getDocFromClasspath(name);
		}
		
		return doc;
	}
	
	/**
	 * This method takes in the name of the documemt that is
	 * to be retrieved and will either return the document if successful or
	 * will return null after an exception is thrown.  This method attempts to use
	 * the other getBinDocFromX() methods of this class to retrieve the given document
	 */
	public static byte[] getBinDoc(String name)
	{
		// try to fetch from URL first:
		
		byte doc[] = getBinDocFromURL(name);
		
		if (doc == null)
		{
			doc = getBinDocFromFile(name);
		}
		
		if (doc == null)
		{
			doc = getBinDocFromClasspath(name);
		}
		
		return doc;
	}
	
	
	public static void main( String args[] )
	{
		System.out.println( "Getting document " + args[0] + " ..." );
		System.out.println( "String = " + getDoc( args[0] ) );
		System.out.println( "Bytes = " + getBinDoc( args[0] ) );
	}
}